@@ -13,31 +13,47 @@
 #-------------------------------------------------------------------------------
 
 import xml.dom.minidom as xml
-import os
+import os, sys, os.path
 import pygst
 pygst.require("0.10")
-import gst
-from Monitored import Monitored
-import Utils
+import gst, gobject
+import Utils, LevelsList
 import UndoSystem
 import Globals
 import gettext
 import urllib
+
+from elements.singledecodebin import SingleDecodeBin
 _ = gettext.gettext
 
 #=========================================================================
 
-class Event(Monitored):
+class Event(gobject.GObject):
 	"""
 	This class handles maintaing the information for a single audio 
 	event, normally, a fragment of a recorded file.
+
+	Signals:
+		"waveform" -- The waveform date for this event has changed.
+		"position" -- The starting position of this event has changed.
+		"length" -- The length of this event has changed.
+		"corrupt" -- The audio file for this event is not playable. Two strings with detailed information are sent.
+		"loading" -- Loading has started or completed.
+
 	"""
-	
-	""" State changed types (to be sent through the Monitored class) """
-	WAVEFORM, MOVE, LENGTH, CORRUPT, LOADING = range(5)
-	
+	__gsignals__ = {
+		"waveform" 	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"position" 	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"length" 		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"corrupt" 	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
+		"loading" 	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"selected" 	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () )
+	}
+
 	""" The level sample interval in seconds """
 	LEVEL_INTERVAL = 0.01
+	LEVELS_FILE_EXTENSION = ".leveldata"
+	NANO_TO_MILLI_DIVISOR = gst.SECOND / 1000
 	
 	#_____________________________________________________________________
 	
@@ -48,19 +64,25 @@ class Event(Monitored):
 		Parameters:
 			instrument -- Instrument associated with this Event.
 			file -- the file this Event should play.
+			levels_file -- the file name (not directory path!) where the event's levels data will be stored
 			id -- unique ID for this Event. If it's taken, a new one will be generated.
 			filelabel -- label to print in error messages.
 						It can be different	from the file parameter.
 		"""
-		Monitored.__init__(self)
+		gobject.GObject.__init__(self)
 		
+		self.id = instrument.project.GenerateUniqueID(id)  #check is id is already taken, then set it.
 		self.start = 0.0			# Time in seconds at which the event begins
 		self.duration = 0.0			# Duration in seconds of the event
 		# The file this event should play (without escaped characters)
 		# If you need characters escaped, please do self.file.replace(" ", "\ ") 
 		# but **do not** assign it to this variable.
 		self.file = file
-
+		
+		# levels_file is a filename only, no directory information for levels here.
+		basename = os.path.basename(self.file or "Unknown")
+		self.levels_file = "%s_%d%s" % (basename, self.id, self.LEVELS_FILE_EXTENSION)
+		
 		# the label is the filename to print in error messages
 		# if it differs from the real filename (i.e its been copied into the project)
 		if filelabel != None:
@@ -72,11 +94,11 @@ class Event(Monitored):
 		self.name = "New Event"		# Name of this event
 		
 		self.selection  = [0, 0]	# List start and end of selection (for fades, etc) measured in seconds 
-		self.levels = []			# Array of audio levels to be drawn for this event
+		self.levels_list = LevelsList.LevelsList()	# LevelsList class containing array of audio levels to be drawn for this event
 		
-		self.id = instrument.project.GenerateUniqueID(id)  #check is id is already taken, then set it.
 		self.instrument = instrument	# The parent instrument
-		self.filesrc = None 			# The gstreamer gnlfilesource object.
+		self.gnlsrc = None 			# The gstreamer gnlsource object.
+		self.single_decode_bin = None		# The gstreamer file decoder element.
 		
 		self.offset = 0.0			# Offset through the file in seconds
 		self.isLoading = False		# True if the event is currently loading level data
@@ -99,9 +121,9 @@ class Event(Monitored):
 		# The list *must* be ordered by time-in-seconds, so when you update it from
 		# the dictionary using dict.items(), be sure to sort it again.
 		self.audioFadePoints = []
-		#Just like self.levels except with all the levels scaled according to the
+		#Just like self.levels_list except with all the levels scaled according to the
 		#points in self.audioFadePoints.
-		self.fadeLevels = []
+		self.fadeLevels = LevelsList.LevelsList()
 
 	#_____________________________________________________________________
 	
@@ -112,12 +134,21 @@ class Event(Monitored):
 		properties.
 		"""
 		Globals.debug("create file source")
-		if not self.filesrc:
-			self.filesrc = gst.element_factory_make("gnlfilesource", "Event_%d"%self.id)
-		if not self.filesrc in list(self.instrument.composition.elements()):
-			self.instrument.composition.add(self.filesrc)
+		if not self.gnlsrc:
+			self.gnlsrc = gst.element_factory_make("gnlsource", "Event_%d"%self.id)
+		if not self.gnlsrc in list(self.instrument.composition.elements()):
+			self.instrument.composition.add(self.gnlsrc)
 		
 		self.SetProperties()
+
+	#_____________________________________________________________________
+
+	def DestroyFilesource(self):
+		"""
+		Removes the Gstreamer file source from the instrument's composition.
+		"""
+		if self.gnlsrc in list(self.instrument.composition.elements()):
+			self.instrument.composition.remove(self.gnlsrc)
 		
 	#_____________________________________________________________________
 		
@@ -126,8 +157,18 @@ class Event(Monitored):
 		Sets basic Event properties like location, start, duration, etc.
 		"""
 		if self.file:
+			if self.single_decode_bin:
+				self.gnlsrc.remove(self.single_decode_bin)
+
+			Globals.debug("creating SingleDecodeBin")
+			caps = gst.caps_from_string("audio/x-raw-int;audio/x-raw-float")
+			f = "file://" + self.file
+			self.single_decode_bin = SingleDecodeBin(caps=caps, uri=f)
+			Globals.debug("file uri is:", f)
+			self.gnlsrc.add(self.single_decode_bin)
 			Globals.debug("setting event properties:")
-			propsDict = {"location" : self.file,
+			propsDict = {
+					"caps" : caps,
 					"start" : long(self.start * gst.SECOND),
 					"duration" : long(self.duration * gst.SECOND),
 					"media-start" : long(self.offset * gst.SECOND),
@@ -136,7 +177,7 @@ class Event(Monitored):
 					}
 					
 			for prop, value in propsDict.iteritems():
-				self.filesrc.set_property(prop, value)
+				self.gnlsrc.set_property(prop, value)
 				Globals.debug("\t", prop, "=", value)
 
 	#_____________________________________________________________________
@@ -163,7 +204,8 @@ class Event(Monitored):
 		ev.appendChild(params)
 		
 		items = ["start", "duration", "isSelected", 
-				  "name", "offset", "file", "isLoading", "isRecording"
+				  "name", "offset", "file", "filelabel", "levels_file",
+				  "isLoading", "isRecording"
 				]
 				
 		#Since we are saving the path to the project file, don't delete it on exit
@@ -171,7 +213,7 @@ class Event(Monitored):
 			self.instrument.project.deleteOnCloseAudioFiles.remove(self.file)
 		
 		self.temp = self.file
-		if os.path.samefile(self.instrument.path, os.path.dirname(self.file)):
+		if os.path.samefile(self.instrument.project.audio_path, os.path.dirname(self.file)):
 			# If the file is in the audio dir, just include the filename, not the absolute path
 			self.file = os.path.basename(self.file)
 		
@@ -185,11 +227,8 @@ class Event(Monitored):
 		ev.appendChild(xmlPoints)
 		Utils.StoreDictionaryToXML(doc, xmlPoints, self.__fadePointsDict, "FadePoint")
 		
-		if self.levels:
-			levelsXML = doc.createElement("Levels")
-			ev.appendChild(levelsXML)
-			stringList = map(str, self.levels)
-			levelsXML.setAttribute("value", ",".join(stringList))
+		if self.levels_list:
+			self.levels_list.tofile(os.path.join(self.instrument.project.levels_path, self.levels_file))
 		
 	#_____________________________________________________________________
 		
@@ -222,39 +261,109 @@ class Event(Monitored):
 	
 	#_____________________________________________________________________
 	
-	@UndoSystem.UndoCommand("Move", "start", "temp")
-	def Move(self, frm, to):
+	@UndoSystem.UndoCommand("Move", "temp")
+	def Move(self, to, frm=None):
 		"""
 		Moves this Event in time.
 
 		Parameters:
-			frm -- the time the Event's moving from.
 			to -- the time the Event's moving to.
+			frm -- the time the Event's moving from.
 		"""
-		self.temp = frm
+		if frm is None:
+			self.temp = self.start
+		else:
+			self.temp = frm
 		self.start = to
 		self.SetProperties()
+		self.emit("position")
 	
 	#_____________________________________________________________________
 	
-	def Split(self, split_point, id=-1):
+	def _Compat09_Move(self, frm, to, _undoAction_):
 		"""
-		Dummy function kept for compatibility with 0.2 project files.
+		Moves this Event in time.
+		
+		Considerations:
+			A compatibility method for undo functions from 
+			0.1, 0.2 and 0.9 project files. It should not be called
+			explicitly by anyone.
+		
+		Parameters:
+			frm -- the time the Event's moving from.
+			to -- the time the Event's moving to.
+		"""
+		self.Move(to, frm, _undoAction_=_undoAction_)
+	
+	#_____________________________________________________________________
+	
+	def _Compat02_Split(self, split_point, id=-1):
+		"""
+		Function kept for compatibility with 0.2 project files.
 		Parameters are the same as SplitEvent().
 		"""
 		self.SplitEvent(split_point)
 	
 	#_____________________________________________________________________
 	
-	def Join(self, joinEventID):
+	def _Compat02_Join(self, joinEventID):
 		"""
-		Dummy function kept for compatibility with 0.2 project files.
+		Function kept for compatibility with 0.2 project files.
 		Parameters are the same as JoinEvent().
 		"""
 		self.JoinEvent(joinEventID)
 	
 	#_____________________________________________________________________
 	
+	def CopySelection(self, eventID=-1):
+		"""
+		Only for use with a 2-point selection.
+		Essentially performs a 'fake split' and returns a new event
+		which would be the result of splitting an event at the 2 points.
+		
+		This is used when the user shift-drags an event to create a selection,
+		then chooses 'copy' from the context menu. The new event can be placed
+		wherever the user wishes by right-clicking and choosing 'paste'.
+		"""
+		if eventID >= 0:
+			e = [x for x in self.instrument.graveyard if x.id == eventID][0]
+			self.instrument.graveyard.remove(e)
+		else:
+			e = Event(self.instrument, self.file)
+		e.name = self.name
+		
+		dur = self.selection[1] - self.selection[0]
+		
+		e.start = self.start + self.selection[0]
+		e.offset = self.selection[0] #+self.offset
+		e.duration = dur
+		
+		dictLeft = {}
+		dictRight = {}
+		for key, value in self.__fadePointsDict.iteritems():
+			if key < self.selection[0]:
+				dictLeft[key] = value
+			if key > self.selection[0]:
+				dictRight[key - self.selection[0]] = value
+		#in case there is a fade passing through the split point, recreate half of it on either side
+		splitFadeLevel = self.GetFadeLevelAtPoint(self.selection[0])
+		dictLeft[self.selection[0]] = splitFadeLevel
+		dictRight[0.0] = splitFadeLevel
+		
+		millis = int(self.selection[0] * 1000)
+		e.levels_list = self.levels_list.slice_by_endtime(millis)
+		e.__fadePointsDict = dictRight
+			
+		e.__UpdateAudioFadePoints()
+		e.SetProperties()
+		self.instrument.events.append(e)
+		e.emit("length")
+		e.emit("position")
+		
+		return e
+		
+
+	
 	@UndoSystem.UndoCommand("JoinEvent", "temp", "temp2")
 	def SplitEvent(self, split_point, cutRightSide=True, eventID=-1):
 		"""
@@ -302,9 +411,9 @@ class Event(Monitored):
 			e.duration = dur - split_point
 			self.duration = split_point
 			
-			nl = int(len(self.levels) * (split_point / dur))
-			e.levels = self.levels[nl:]
-			self.levels = self.levels[:nl]
+			millis = int(split_point * 1000)
+			e.levels_list = self.levels_list.slice_by_endtime(millis)
+			self.levels_list = self.levels_list.slice_by_endtime(0, millis)
 			
 			self.__fadePointsDict = dictLeft
 			e.__fadePointsDict = dictRight
@@ -317,9 +426,9 @@ class Event(Monitored):
 			self.offset = self.offset + split_point
 			self.duration = dur - split_point
 			
-			nl = int(len(self.levels) * (split_point / dur))
-			e.levels = self.levels[:nl]
-			self.levels = self.levels[nl:]
+			millis = int(split_point * 1000)
+			e.levels_list = self.levels_list.slice_by_endtime(0, millis)
+			self.levels_list = self.levels_list.slice_by_endtime(millis)
 			
 			self.__fadePointsDict = dictRight
 			e.__fadePointsDict = dictLeft
@@ -329,8 +438,9 @@ class Event(Monitored):
 		e.SetProperties()
 		self.instrument.events.append(e)
 		self.SetProperties()
-		self.StateChanged(self.LENGTH)
-		self.StateChanged(self.MOVE)
+		self.emit("length")
+		self.emit("position")
+		self.instrument.emit("event::added", e)
 		
 		#undo parameters
 		self.temp = e.id
@@ -362,8 +472,9 @@ class Event(Monitored):
 			if self.__fadePointsDict.has_key(self.duration):
 				del self.__fadePointsDict[self.duration]
 			
+			old_duration = int(self.duration * 1000)
 			self.duration += joinEvent.duration
-			self.levels.extend(joinEvent.levels)
+			self.levels_list.extend(old_duration, joinEvent.levels_list)
 			#update the fade point list after the level, and duration because it depends on them
 			self.__UpdateAudioFadePoints()
 		else:
@@ -371,8 +482,10 @@ class Event(Monitored):
 		
 			self.start = joinEvent.start
 			self.offset = joinEvent.offset
+			
+			old_duration = int(self.duration * 1000)
 			self.duration += joinEvent.duration
-			self.levels = joinEvent.levels + self.levels
+			self.levels_list = LevelsList.add(old_duration, joinEvent.levels_list, self.levels_list)
 			
 			newDict = joinEvent.__fadePointsDict.copy()
 			for key, value in self.__fadePointsDict.iteritems():
@@ -383,14 +496,14 @@ class Event(Monitored):
 			self.__fadePointsDict = newDict
 			self.__UpdateAudioFadePoints()
 			
+		#create an undo action that is not attached to the project so that
+		# the following delete will not be undone (it will be re-split not resurrected)
+		nullAction = UndoSystem.AtomicUndoAction()
 		# Now that they're joined, move delete the rightEvent
-		if joinEvent in self.instrument.events:
-			self.instrument.events.remove(joinEvent)
-		if not joinEvent in self.instrument.graveyard:
-			self.instrument.graveyard.append(joinEvent)
+		joinEvent.Delete(_undoAction_=nullAction)
 		
-		self.StateChanged(self.LENGTH)
-		self.StateChanged(self.MOVE)
+		self.emit("length")
+		self.emit("position")
 		
 		self.temp2 = joinToRight
 		self.temp3 = joinEvent.id
@@ -427,11 +540,11 @@ class Event(Monitored):
 			self.instrument.DeleteEvent(rightSplit.id, _undoAction_=undoAction)
 		
 		self.SetProperties()
-		self.StateChanged(self.LENGTH)
+		self.emit("length")
 		
 	#_____________________________________________________________________
 	
-	def UndoTrim(self, leftID, rightID):
+	def _Compat02_UndoTrim(self, leftID, rightID):
 		"""
 		Resurrects two pieces from the graveyard and joins them to
 		either side of this Event.
@@ -479,6 +592,19 @@ class Event(Monitored):
 			self.instrument.ResurrectEvent(self.id)
 			
 	#_____________________________________________________________________
+
+	def install_plugin_cb(self, result):
+		self._installing_plugins = False
+		if result == gst.pbutils.INSTALL_PLUGINS_SUCCESS:
+			gst.update_registry()
+			self.GenerateWaveform()
+			return
+
+		# FIXME: send a better error
+		msg = "failed to install plugins: %s" % result
+		self.emit("corrupt", msg)
+
+	#_____________________________________________________________________
 	
 	def bus_message(self, bus, message):
 		"""
@@ -498,18 +624,27 @@ class Event(Monitored):
 			return False
 
 		st = message.structure
-		if st:
-			if st.get_name() == "level":
-				newLevel = self.__CalculateAudioLevel(st["peak"])
-				self.levels.append(newLevel)
-				
-				end = st["endtime"] / float(gst.SECOND)
-				self.loadingLength = int(end)
-				
-				# Only send events every second processed to reduce GUI load
-				if self.loadingLength != self.lastEnd:
-					self.lastEnd = self.loadingLength 
-					self.StateChanged(self.LENGTH) # tell the GUI
+		if not st:
+			return False
+		
+		if st.get_name().startswith('missing-'):
+			self.loadingPipeline.set_state(gst.STATE_NULL)
+			Utils.HandleGstPbutilsMissingMessage(message, self.install_plugin_cb)
+
+		elif st.get_name() == "level":
+			# FIXME: currently everything is being averaged to a single channel
+			peak = Utils.CalculateAudioLevel(st["rms"])
+			#convert number from gst.SECOND (i.e. nanoseconds) to milliseconds
+			end = int(st["endtime"] / self.NANO_TO_MILLI_DIVISOR)
+			self.levels_list.append(end,  [peak])
+			
+			#Truncate at 1000 milliseconds so it updates once per second
+			self.loadingLength = end / 1000
+			
+			# Only send events every second processed to reduce GUI load
+			if self.loadingLength != self.lastEnd:
+				self.lastEnd = self.loadingLength 
+				self.emit("length") # tell the GUI
 		return True
 		
 	#_____________________________________________________________________
@@ -541,16 +676,25 @@ class Event(Monitored):
 				else:
 					self.duration = self.loadingLength
 			
-			if length and (self.offset > 0 or self.duration != length):
-				dt = int(self.duration * len(self.levels) / length)
-				start = int(self.offset * len(self.levels) / length)
-				self.levels = self.levels[start:start+dt]
+			if self.levels_list:
+				final_endtime = self.levels_list[-1][0]
+				if final_endtime > int(self.duration * 1000):
+					Globals.debug("Event %d: duration (%f) is less than last level endtime (%d)."
+					              % (self.id, self.duration, final_endtime))
+					self.duration = final_endtime / 1000.0
+					self.SetProperties()
+					Globals.debug("\tduration has been increased to", self.duration)
 			
+			if length and (self.offset > 0 or self.duration != length):
+				starttime = int(self.offset * 1000)
+				stoptime = int((self.offset + self.duration) * 1000)
+				self.levels_list = self.levels_list.slice_by_endtime(starttime, stoptime)
+				
 			# We're done with the bin so release it
 			self.StopGenerateWaveform()
 			
 			# Signal to interested objects that we've changed
-			self.StateChanged(self.WAVEFORM)
+			self.emit("waveform")
 			return False
 			
 	#_____________________________________________________________________
@@ -566,14 +710,14 @@ class Event(Monitored):
 		# state has changed
 		try:
 			time = self.loadingPipeline.query_duration(gst.FORMAT_TIME)
-			if self.duration == 0 and time[0] != 0:
+			if self.duration == 0 and time[0] > 0:
 				self.duration = float(time[0] / float(gst.SECOND))
 				
 				#update position with proper duration
 				self.MoveButDoNotOverlap(self.start)
 				self.SetProperties()
-				self.StateChanged(self.MOVE)
-				self.StateChanged(self.LENGTH)
+				self.emit("length")
+				self.emit("position")
 		except:
 			# no size available yet
 			pass
@@ -590,8 +734,13 @@ class Event(Monitored):
 		"""
 		error, debug = message.parse_error()
 		
+		Globals.debug("Event Bus Error Message:")
+		Globals.debug("\tCode:", error.code)
+		Globals.debug("\tDomain:", error.domain)
+		Globals.debug("\tMessage:", error.message)
+		
 		Globals.debug("Event bus error:", str(error), str(debug))
-		self.StateChanged(self.CORRUPT, str(error), str(debug))
+		self.emit("corrupt", "%s\n%s" % (error, debug))
 	
 	#_____________________________________________________________________
 	
@@ -603,7 +752,7 @@ class Event(Monitored):
 			bus -- GStreamer bus sending the message.
 			message -- GStreamer message.
 		"""
-		Globals.debug("recieved group of tags")
+		Globals.debug("received group of tags")
 		st = message.structure
 		
 		title, artist = None, None
@@ -624,7 +773,8 @@ class Event(Monitored):
 		"""
 		Renders the level information for the GUI.
 		"""
-		pipe = """filesrc name=src location=%s ! decodebin ! audioconvert ! level interval=%d message=true ! fakesink""" 
+		pipe = """filesrc name=src location=%s ! decodebin ! audioconvert ! level interval=%d message=true ! fakesink"""
+		
 		pipe = pipe % (self.file.replace(" ", "\ "), self.LEVEL_INTERVAL * gst.SECOND)
 		self.loadingPipeline = gst.parse_launch(pipe)
 
@@ -636,9 +786,9 @@ class Event(Monitored):
 		self.bus.connect("message::eos", self.bus_eos)
 		self.bus.connect("message::error", self.bus_error)
 
-		self.levels = []
+		self.levels_list = LevelsList.LevelsList()
 		self.isLoading = True
-		self.StateChanged(self.LOADING)
+		self.emit("loading")
 
 		self.loadingPipeline.set_state(gst.STATE_PLAYING)
 
@@ -666,9 +816,9 @@ class Event(Monitored):
 		self.bus.connect("message::eos", self.bus_eos)
 		self.bus.connect("message::error", self.bus_error)
 
-		self.levels = []
+		self.levels_list = LevelsList.LevelsList()
 		self.isLoading = True
-		self.StateChanged(self.LOADING)
+		self.emit("loading")
 
 		self.loadingPipeline.set_state(gst.STATE_PLAYING)
 		
@@ -700,7 +850,7 @@ class Event(Monitored):
 			
 			self.loadingPipeline = None
 			self.loadingLength = 0
-			self.StateChanged(self.LOADING)
+			self.emit("loading")
 	
 	#_____________________________________________________________________
 
@@ -723,52 +873,24 @@ class Event(Monitored):
 		
 		st = message.structure
 		if st and message.src.get_name() == "recordlevel":
-			newLevel = self.__CalculateAudioLevel(st["peak"])
-			self.levels.append(newLevel)
+			# FIXME: currently everything is being averaged to a single channel
+			peak = Utils.CalculateAudioLevel(st["rms"])
+			#convert number from gst.SECOND (i.e. nanoseconds) to milliseconds
+			end = int(st["endtime"] / self.NANO_TO_MILLI_DIVISOR)
+			self.levels_list.append(end,  [peak])
 			
-			end = st["endtime"] / float(gst.SECOND)
+			end = end / 1000.0	#convert to float representing seconds 
 			#Round to one decimal place so it updates 10 times per second
 			self.loadingLength = round(end, 1)
 			
 			# Only send events every second processed to reduce GUI load
 			if self.loadingLength != self.lastEnd:
 				self.lastEnd = self.loadingLength 
-				self.StateChanged(self.LENGTH) # tell the GUI
+				self.emit("length") # tell the GUI
 		return True
 		
 	#_____________________________________________________________________
 	
-	def __CalculateAudioLevel(self, channelLevels):
-		"""
-		Calculates an average for all channel levels.
-		
-		Parameters:
-			channelLevels -- list of levels from each channel.
-			
-		Returns:
-			an average level, also taking into account negative infinity numbers,
-			which will be discarded in the average.
-		"""
-		negInf = float("-inf")
-		peaktotal = 0
-		peakcount = 0
-		for peak in channelLevels:
-			#don't add -inf values cause 500 + -inf is still -inf
-			if peak != negInf:
-				peaktotal += peak
-				peakcount += 1
-		#avoid a divide by zero here
-		if peakcount > 0:
-			peaktotal /= peakcount
-		#it must be put back to -inf if nothing has been added to it, so that the DbToFloat conversion will work
-		elif peakcount == 0:
-			peaktotal = negInf
-		
-		#convert to 0...1 float, and return
-		return Utils.DbToFloat(peaktotal)
-		
-	#_____________________________________________________________________
-	
 	def SetSelected(self, sel):
 		"""
 		Enables or disables the selection state for this Event.
@@ -778,10 +900,10 @@ class Event(Monitored):
 					True = the Event has been selected.
 					False = the Event has been deselected.
 		"""
-		# No need to call StateChanged when there is no change in selection state
+		# No need to emit a signal when there is no change in selection state
 		if self.isSelected is not sel:
 			self.isSelected = sel
-			self.StateChanged()
+			self.emit("selected")
 	
 	#_____________________________________________________________________
 	
@@ -958,7 +1080,7 @@ class Event(Monitored):
 				self.audioFadePoints.append(last)
 			
 		self.__UpdateFadeLevels()
-		self.StateChanged(self.WAVEFORM)
+		self.emit("waveform")
 	
 	#_____________________________________________________________________
 	
@@ -969,41 +1091,58 @@ class Event(Monitored):
 		list is a cache of faded levels as they will be shown on the screen
 		so that we don't have to calculate them everytime we draw.
 		"""
-		if not self.audioFadePoints:
+		if not self.audioFadePoints or len(self.audioFadePoints) < 2:
+			Globals.debug("Event", self.id, ": no fade points to use")
 			#there are no fade points for us to use
 			return
 			
-		fadePercents = []
-		oneSecondInLevels = len(self.levels) / self.duration
+		#fadePercents = []
+		#oneSecondInLevels = len(self.levels) / self.duration
 		
-		previousFade = self.audioFadePoints[0]
-		for fade in self.audioFadePoints[1:]:
-			#calculate the number of levels that should be in the list between the two points
-			levelsInThisSection = int(round((fade[0] - previousFade[0]) * oneSecondInLevels))
-			if fade[1] == previousFade[1]:
-				# not actually a fade, just two points at the same volume
-				fadePercents.extend([ fade[1] ] * levelsInThisSection)
-			else:
-				step = (fade[1] - previousFade[1]) / levelsInThisSection
-				floatList = Utils.floatRange(previousFade[1], fade[1], step)
-				#make sure the list of levels does not exceed the calculated length
-				floatList = floatList[:levelsInThisSection]
-				fadePercents.extend(floatList)
-			previousFade = fade
+		self.fadeLevels = LevelsList.LevelsList()
 		
-		if len(fadePercents) != len(self.levels):
-			#make sure its not longer than the levels list
-			fadePercents = fadePercents[:len(self.levels)]
-			#make sure its not shorter than the levels list
-			#by copying the last level over again
-			lastLevel = fadePercents[-1]
-			while len(fadePercents) < len(self.levels):
-				fadePercents.append(lastLevel)
+		iterFadePoints = iter(self.audioFadePoints)
+		firstFadeTime, firstFadeValue = iterFadePoints.next()
+		firstFadeTime = int(firstFadeTime * 1000)	#convert to milliseconds
+		secondFadeTime, secondFadeValue = iterFadePoints.next()
+		secondFadeTime = int(secondFadeTime * 1000)	#convert to milliseconds
+		# if less than one percent difference, assume they are the same
+		sameValues = abs(firstFadeValue - secondFadeValue) < 0.01
+		
+		if not sameValues:
+			slope = (secondFadeValue - firstFadeValue) / (secondFadeTime - firstFadeTime)
+		
+		for endtime, peak in self.levels_list:
+			# check if we have moved into the next fade point pair
+			# don't care about 1 millisecond difference, its rounding error
+			if endtime > (secondFadeTime + 1):  
+				firstFadeTime = secondFadeTime
+				firstFadeValue = secondFadeValue
+				try:
+					secondFadeTime, secondFadeValue = iterFadePoints.next()
+				except StopIteration:
+					Globals.debug("Event %d: endtime (%d) is after last fade point (%d,%d)"
+					              % (self.id, endtime, secondFadeTime, secondFadeValue))
 				
-		self.fadeLevels = []
-		for i in range(len(self.levels)):
-			self.fadeLevels.append(fadePercents[i] * self.levels[i])
-		
+				secondFadeTime = int(secondFadeTime * 1000)	#convert to milliseconds
+				
+				# if less than one percent difference, assume they are the same
+				sameValues = abs(firstFadeValue - secondFadeValue) < 0.01
+				if not sameValues:
+					# the fade line is not flat, so calculate the slope of it
+					slope = (secondFadeValue - firstFadeValue) / (secondFadeTime - firstFadeTime)
+			
+			if sameValues:
+				#no fade here, the same volume continues across
+				self.fadeLevels.append(endtime, [int(peak * firstFadeValue)])
+			else:
+				rel_time = endtime - firstFadeTime
+				peak_delta = slope * rel_time
+				new_fade_value = firstFadeValue + peak_delta
+				assert new_fade_value <= 1.0		# a fade cannot be more than 100%
+				peak = int(peak * new_fade_value)
+				
+				self.fadeLevels.append(endtime, [peak])
 	#_____________________________________________________________________
 	
 	def GetFadeLevels(self):
@@ -1018,10 +1157,11 @@ class Event(Monitored):
 		"""
 		# no fades registered
 		if not self.audioFadePoints:
-			return self.levels[:]
+			return self.levels_list
 			
-		if len(self.fadeLevels) != len(self.levels):
+		if len(self.fadeLevels) != len(self.levels_list):
 			self.__UpdateFadeLevels()
+			assert len(self.fadeLevels) == len(self.levels_list)
 			
 		return self.fadeLevels
 		
