@@ -68,9 +68,11 @@ class EventLaneViewer(gtk.EventBox):
 			
 		self.project = project
 		self.instrument = instrument
-		self.project.transport.AddListener(self)
-		self.project.AddListener(self)
-		self.instrument.AddListener(self)
+		self.project.transport.connect("position", self.OnTransportPosition)
+		self.project.connect("view-start", self.OnProjectViewChange)
+		self.project.connect("zoom", self.OnProjectViewChange)
+		self.instrument.connect("event::added", self.OnEventAdded)
+		self.instrument.connect("event::removed", self.OnEventRemoved)
 		
 		# This defines where the blue cursor indicator should be drawn (in pixels)
 		self.highlightCursor = None
@@ -81,6 +83,9 @@ class EventLaneViewer(gtk.EventBox):
 		#The position where the last mouse click was
 		self.mouseDownPos = [0,0]
 		
+		#the list of all the EventViewer widgets
+		self.eventViewerList = []
+		
 		self.set_events(	gtk.gdk.POINTER_MOTION_MASK |
 							gtk.gdk.BUTTON_RELEASE_MASK |
 							gtk.gdk.BUTTON_PRESS_MASK )
@@ -96,8 +101,35 @@ class EventLaneViewer(gtk.EventBox):
 		self.fixed.connect("drag_leave", self.OnDragLeave)
 		self.fixed.connect("expose-event", self.OnDraw)
 		
+		#create the context menu
+		self.contextMenu = gtk.Menu()
+		
+		audioimg = gtk.Image()
+		if self.mainview.audioFilePixbuf:
+			size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
+			pixbuf = self.mainview.audioFilePixbuf.scale_simple(size[0], size[1], gtk.gdk.INTERP_BILINEAR)
+			audioimg.set_from_pixbuf(pixbuf)
+		
+		menuItem = gtk.ImageMenuItem(_("_Add Audio File..."), True)
+		menuItem.set_image(audioimg)
+		menuItem.connect("activate", self.CreateEventFromFile)
+		self.contextMenu.append(menuItem)
+		
+		menuItem = gtk.SeparatorMenuItem()
+		self.contextMenu.append(menuItem)
+		
+		self.pasteContextMenuItem = gtk.ImageMenuItem(gtk.STOCK_PASTE)
+		self.pasteContextMenuItem.connect("activate", self.OnPaste)
+		self.contextMenu.append(self.pasteContextMenuItem)
+		
+		menuItem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
+		menuItem.connect("activate", self.OnDelete)
+		self.contextMenu.append(menuItem)
+		
 		self.messageID = None
-		self.Update()
+		
+		for event in self.instrument.events:
+			self.OnEventAdded(self.instrument, event)
 		
 	#_____________________________________________________________________
 		
@@ -136,45 +168,18 @@ class EventLaneViewer(gtk.EventBox):
 		
 	#_____________________________________________________________________
 		
-	def Update(self, child=None):
+	def UpdatePosition(self, eventViewer):
 		"""
-		Updates the complete view when requested by OnStateChanged or __init__.
+		Moves the given EventViewer widget to the appropriate position.
 		
 		Parameters:
-			child -- a particular child widget to be updated. If provided, only this 
-					one will be updated, and all other children will be left alone.
+			eventViewer -- the widget that has needs to be moved to a new position.
 		"""
-		if child and child in self.fixed.get_children():
-			x = int(round((child.event.start - self.project.viewStart) * self.project.viewScale))
-			self.fixed.move( child, x, 0 )
-			child.UpdateDrawerPosition()
-		else:			
-			# Move them to the correct positions
-			for widget in self.fixed.get_children():
-				#Check that it is EventViewer (could be a button drawer)
-				if type(widget) == EventViewer:
-					if widget.event not in self.instrument.events:
-						# Check if any events have been deleted
-						self.fixed.remove(widget)
-						# remove the event's drawer if it's showing
-						if widget.drawer.parent == self.fixed:
-							self.fixed.remove(widget.drawer)
-						#destroy the object
-						widget.Destroy()
-					else:
-						x = int(round((widget.event.start - self.project.viewStart) * self.project.viewScale))
-						self.fixed.move(widget, x, 0)
-						widget.UpdateDrawerPosition()
-
-			# Check if any events have been added
-			widget_events = [widget.event for widget in self.fixed.get_children()]
-			for ev in self.instrument.events:
-				if ev not in widget_events:
-					x = int(round((ev.start - self.project.viewStart) * self.project.viewScale))
-					child = EventViewer(self, self.project, ev, self.allocation.height, self.mainview, self.small)
-					self.fixed.put(child, x, 0)
-			self.fixed.show_all()
-		self.queue_draw()
+		if eventViewer in self.fixed.get_children():
+			x = int(round((eventViewer.event.start - self.project.viewStart) * self.project.viewScale))
+			self.fixed.move( eventViewer, x, 0 )
+			
+			self.queue_draw()
 			
 	#_____________________________________________________________________
 	
@@ -182,11 +187,12 @@ class EventLaneViewer(gtk.EventBox):
 		"""
 		Called when the EventLaneViewer gets destroyed.
 		It also destroys any child widget and disconnects itself from any
-		listening objects via Monitored.
+		gobject signals.
 		"""
-		self.project.transport.RemoveListener(self)
-		self.project.RemoveListener(self)
-		self.instrument.RemoveListener(self)
+		self.project.transport.disconnect_by_func(self.OnTransportPosition)
+		self.project.disconnect_by_func(self.OnProjectViewChange)
+		self.instrument.disconnect_by_func(self.OnEventAdded)
+		self.instrument.disconnect_by_func(self.OnEventRemoved)
 		
 		for widget in self.fixed.get_children():
 			#Check that it is EventViewer (could be a button drawer)
@@ -219,38 +225,14 @@ class EventLaneViewer(gtk.EventBox):
 			self.OnMenuDone()
 		# Create context menu on RMB 
 		elif mouse.button == 3:
-			menu = gtk.Menu()
+			self.pasteContextMenuItem.set_sensitive( bool(self.project.clipboardList) )
 		
-			audioimg = None
-			if self.mainview.audioFilePixbuf:
-				audioimg = gtk.Image()
-				audioimg.set_from_pixbuf(self.mainview.audioFilePixbuf)
-			
-			items = [	(_("_Add Audio File..."), self.CreateEventFromFile, True, audioimg),
-					("---", None, None, None),
-					(_("_Paste"), self.OnPaste, self.project.clipboardList, gtk.image_new_from_stock(gtk.STOCK_PASTE, gtk.ICON_SIZE_MENU)),
-					(_("_Delete"), self.OnDelete, True, gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU))
-					]
-
-			for label, callback, sensitive, image in items:
-				if label == "---":
-					menuItem = gtk.SeparatorMenuItem()
-				elif image:
-					menuItem = gtk.ImageMenuItem(label, True)
-					menuItem.set_image(image)
-				else:
-					menuItem = gtk.MenuItem(label=label)
-					
-				menuItem.set_sensitive(bool(sensitive))
-				menuItem.show()
-				menu.append(menuItem)
-				if callback:
-					menuItem.connect("activate", callback)
 			self.highlightCursor = mouse.x
 			self.popupIsActive = True
-
-			menu.popup(None, None, None, mouse.button, mouse.time)
-			menu.connect("selection-done", self.OnMenuDone)
+			
+			self.contextMenu.connect("selection-done", self.OnMenuDone)
+			self.contextMenu.show_all()
+			self.contextMenu.popup(None, None, None, mouse.button, mouse.time)
 		
 		if 'GDK_CONTROL_MASK' in mouse.state.value_names:
 			self.instrument.SetSelected(True)
@@ -365,32 +347,77 @@ class EventLaneViewer(gtk.EventBox):
 			event -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		self.project.DeleteInstrument(self.instrument.id)
-		self.mainview.UpdateDisplay()
 	
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None, *extra):
+	def OnProjectViewChange(self, project):
 		"""
-		Called when a change of state is signalled by any of the
-		objects this view is 'listening' to.
-		If there's a Project or Instrument change, then redraw everything,
-		otherwise just redraw the playhead.
+		Callback function for when the project view changes,
+		and the "view-start" or the "zoom" signal is send, and we
+		need to update.
 		
 		Parameters:
-			obj -- object changing state.
-			change -- the change which has occured.
-			extra -- extra parameters passed by the caller.
+			project -- The project instance that send the signal.
 		"""
-		if obj is self.project or obj is self.instrument:
-			self.Update()
-		else:
-			x1 = round((self.project.transport.PrevPosition - self.project.viewStart) * self.project.viewScale)
-			x2 = round((self.project.transport.position - self.project.viewStart) * self.project.viewScale)
-			self.queue_draw_area(int(x1)-1, 0, 3, self.allocation.height)
-			self.queue_draw_area(int(x2)-1, 0, 3, self.allocation.height)
+		for event in self.eventViewerList:
+			self.UpdatePosition(event)
 		
 	#_____________________________________________________________________
 	
+	def OnEventAdded(self, instrument, event):
+		"""
+		Callback for when an event is added to our instrument.
+		
+		Parameters:
+			instrument -- the instrument instance that send the signal.
+			event -- the event instance that was added.
+		"""
+		x = int(round((event.start - self.project.viewStart) * self.project.viewScale))
+		child = EventViewer(self, self.project, event, self.allocation.height, self.mainview, self.small)
+		self.fixed.put(child, x, 0)
+		child.show()
+		self.eventViewerList.append(child)
+	
+	#_____________________________________________________________________
+	
+	def OnEventRemoved(self, instrument, event):
+		"""
+		Callback for when an event is removed from our instrument.
+		
+		Parameters:
+			instrument -- the instrument instance that send the signal.
+			event -- the event instance that was removed.
+		"""
+		for widget in self.eventViewerList:
+			if widget.event is event:
+				self.fixed.remove(widget)
+				# remove the event's drawer if it's showing
+				if widget.drawer.parent == self.fixed:
+					self.fixed.remove(widget.drawer)
+				#destroy the object
+				widget.Destroy()
+				self.eventViewerList.remove(widget)
+				break
+	
+	#_____________________________________________________________________
+	
+	def OnTransportPosition(self, transportManager, extraString):
+		"""
+		Callback for signal when the transport position changes.
+		Here we just redraw the playhead.
+		
+		Parameters:
+			transportManager -- the TransportManager instance that send the signal.
+			extraString -- a string specifying the extra action details. i.e. "stop-action"
+					means that the position changed because the user hit stop.
+		"""
+		x1 = round((self.project.transport.PrevPosition - self.project.viewStart) * self.project.viewScale)
+		x2 = round((self.project.transport.position - self.project.viewStart) * self.project.viewScale)
+		self.queue_draw_area(int(x1)-1, 0, 3, self.allocation.height)
+		self.queue_draw_area(int(x2)-1, 0, 3, self.allocation.height)
+	
+	#_____________________________________________________________________
+	
 	def OnDragDataReceived(self, widget, context, x, y, selection, targetType, time):
 		"""
 		Called when the user releases MOUSE1, finishing a drag and drop
@@ -455,5 +482,59 @@ class EventLaneViewer(gtk.EventBox):
 		"""
 		self.highlightCursor = None
 		self.queue_draw()
+		
+	#_____________________________________________________________________
 	
+	def ChangeSize(self, small):
+		"""
+		Changes the size of the event lane.
+		
+		Parameters:
+			small - True if the event lane is to be small.
+		"""
+		self.small = small
+		for widget in self.fixed.get_children():
+			if type(widget) == EventViewer:
+				widget.ChangeSize(small)
+
+				if widget.drawer in self.fixed.get_children():
+					widget.ShowDrawer()
+	
+	#____________________________________________________________________	
+	
+	def PutDrawer(self, drawer, xvalue=1):
+		"""
+		Places the drawer below in the event lane and makes it visible.
+		
+		Parameters:
+			drawer -- the widget to show.
+			xvalue -- the horizontal position of the drawer in pixels
+		"""
+		if self.small:
+		    yvalue = 30
+		else:
+		    yvalue = 75
+
+		if not drawer.parent:
+			self.fixed.put(drawer, xvalue, yvalue)
+		elif drawer.parent == self.fixed:
+			self.fixed.move(drawer, xvalue, yvalue)
+		
+		drawer.show_all()
+	
+	#____________________________________________________________________
+	
+	def RemoveDrawer(self, drawer):
+		"""
+		Removes the drawer from below in the event. This function does
+		nothing if the given drawer is not currenly shown.
+		
+		Parameters:
+			drawer -- the widget to remove.
+		"""
+		if drawer.parent == self.fixed:
+			self.fixed.remove(drawer)
+			
+	#____________________________________________________________________
+
 #=========================================================================
