@@ -15,10 +15,11 @@ import gtk
 import cairo
 from Project import Project
 import Utils
-import os
+import os, sys
 import gettext
 _ = gettext.gettext
 import Globals
+import itertools
 
 #=========================================================================
 
@@ -81,12 +82,12 @@ class EventViewer(gtk.DrawingArea):
 		
 		gtk.DrawingArea.__init__(self)
 
-		self.set_events(	gtk.gdk.POINTER_MOTION_MASK |
-							gtk.gdk.BUTTON_RELEASE_MASK |
-							gtk.gdk.BUTTON_PRESS_MASK |
-							gtk.gdk.LEAVE_NOTIFY_MASK |
-							gtk.gdk.KEY_PRESS_MASK)
-							
+		self.set_events(gtk.gdk.POINTER_MOTION_MASK |
+		                gtk.gdk.BUTTON_RELEASE_MASK |
+		                gtk.gdk.BUTTON_PRESS_MASK |
+		                gtk.gdk.LEAVE_NOTIFY_MASK |
+		                gtk.gdk.KEY_PRESS_MASK)
+		
 		self.connect("expose_event",self.OnDraw)
 		self.connect("motion_notify_event", self.OnMouseMove)
 		self.connect("leave_notify_event", self.OnMouseLeave)
@@ -108,23 +109,32 @@ class EventViewer(gtk.DrawingArea):
 		self.lane = lane				# The parent lane for this object
 		self.currentScale = 0			# Tracks if the project viewScale has changed
 		self.redrawWaveform = False		# Force redraw the cached waveform on next expose event
-		self.drawerAlignToLeft = True		#boolean; if the drawer should be at the left of current selection
-									#otherwise it will be put on the right
+		#boolean; if the drawer should be at the left of current selection
+		#otherwise it will be put on the right
+		self.drawerAlignToLeft = True		
+		
 		self.fadeMarkers = [100,100]		#the values of the right and left fade markers on the selection
 
 		# Set accessibility helpers
 		self.SetAccessibleName()
 		self.set_property("can-focus", True)
 		
-		# source is an offscreen canvas to hold our waveform image
-		self.source = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
+		# sourceSmall/Large are offscreen canvases to hold our waveform images
+		self.sourceSmall = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
+		self.sourceLarge = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
 		
-		# rectangle of cached draw area
-		self.cachedDrawArea = gtk.gdk.Rectangle(0, 0, 0, 0)
+		# rectangle of cached draw areas
+		self.cachedDrawAreaSmall = gtk.gdk.Rectangle(0, 0, 0, 0)
+		self.cachedDrawAreaLarge = gtk.gdk.Rectangle(0, 0, 0, 0)
 
 		# Monitor the things this object cares about
-		self.project.AddListener(self)
-		self.event.AddListener(self)
+		self.project.connect("zoom", self.OnProjectZoom)
+		self.event.connect("waveform", self.OnEventWaveform)
+		self.event.connect("position", self.OnEventPosition)
+		self.event.connect("length", self.OnEventLength)
+		self.event.connect("corrupt", self.OnEventCorrupt)
+		self.event.connect("loading", self.OnEventLoading)
+		self.event.connect("selected", self.OnEventSelected)
 
 		# This defines where the blue cursor indicator should be drawn (in pixels)
 		self.highlightCursor = None
@@ -182,18 +192,29 @@ class EventViewer(gtk.DrawingArea):
 		Returns:
 			False -- stop propagating the GTK signal. *CHECK*
 		"""
-		cache = self.cachedDrawArea
+		if self.small:
+			cache = self.cachedDrawAreaSmall
+			source = self.sourceSmall
+		else:
+			cache = self.cachedDrawAreaLarge
+			source = self.sourceLarge
 		area = event.area
 		
 		#check if the expose area is within the already cached rectangle
 		if area.x < cache.x or (area.x + area.width > cache.x + cache.width) or self.redrawWaveform:
 			self.DrawWaveform(event.area)
+			if self.small:
+				cache = self.cachedDrawAreaSmall
+				source = self.sourceSmall
+			else:
+				cache = self.cachedDrawAreaLarge
+				source = self.sourceLarge
 		
 		# Get a cairo surface for this drawing op
 		context = widget.window.cairo_create()
 
 		# Give it our waveform image as a source
-		context.set_source_surface(self.source, self.cachedDrawArea.x, self.cachedDrawArea.y)	
+		context.set_source_surface(source, cache.x, cache.y)	
 
 		# Blit our waveform across
 		context.paint()
@@ -263,7 +284,7 @@ class EventViewer(gtk.DrawingArea):
 				pixxFM_left -= self._PIXX_FADEMARKER_WIDTH
 			pixyFM_left = int(padded_height * (100-self.fadeMarkers[0]) / 100.0)
 			context.rectangle(pixxFM_left, pixyFM_left,
-							self._PIXX_FADEMARKER_WIDTH , self._PIXY_FADEMARKER_HEIGHT)
+			                  self._PIXX_FADEMARKER_WIDTH , self._PIXY_FADEMARKER_HEIGHT)
 			
 			pixxFM_right = x2
 			
@@ -273,7 +294,7 @@ class EventViewer(gtk.DrawingArea):
 				pixxFM_right -= self._PIXX_FADEMARKER_WIDTH
 			pixyFM_right = int(padded_height * (100-self.fadeMarkers[1]) / 100.0)
 			context.rectangle(pixxFM_right, pixyFM_right,
-							self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
+			                  self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
 			
 			context.fill()
 			
@@ -286,9 +307,9 @@ class EventViewer(gtk.DrawingArea):
 			
 			# redo the rectangles so they're the path and we can in_fill() check later
 			context.rectangle(pixxFM_left, pixyFM_left,
-							self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
+			                  self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
 			context.rectangle(pixxFM_right, pixyFM_right,
-							self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
+					          self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
 			self.fadeMarkersContext = context
 
 		return False
@@ -305,7 +326,7 @@ class EventViewer(gtk.DrawingArea):
 		allocArea = self.get_allocation()
 		
 		rect = gtk.gdk.Rectangle(exposeArea.x - exposeArea.width, exposeArea.y,
-						exposeArea.width*3, exposeArea.height)
+		                         exposeArea.width*3, exposeArea.height)
 		
 		#Check if our area to cache is outside the allocated area
 		if rect.x < 0:
@@ -313,9 +334,18 @@ class EventViewer(gtk.DrawingArea):
 		if rect.x + rect.width > allocArea.width:
 			rect.width = allocArea.width - rect.x
 			
-		self.source = cairo.ImageSurface(cairo.FORMAT_ARGB32, rect.width, rect.height)
+		#set area to record where the cached surface goes
+		if self.small:
+			self.cachedDrawAreaSmall = rect
+			self.sourceSmall = cairo.ImageSurface(cairo.FORMAT_ARGB32, 
+			                                      rect.width, rect.height)
+			context = cairo.Context(self.sourceSmall)
+		else:
+			self.cachedDrawAreaLarge = rect
+			self.sourceLarge = cairo.ImageSurface(cairo.FORMAT_ARGB32, 
+			                                      rect.width, rect.height)
+			context = cairo.Context(self.sourceLarge)
 		
-		context = cairo.Context(self.source)
 		context.set_line_width(2)
 		context.set_antialias(cairo.ANTIALIAS_SUBPIXEL)
 
@@ -324,29 +354,40 @@ class EventViewer(gtk.DrawingArea):
 		context.set_source_rgb(*self._BACKGROUND_RGB)
 		context.fill()
 		
-		if self.event.levels and (self.event.duration or self.event.loadingLength):
+		if self.event.levels_list and (self.event.duration or self.event.loadingLength):
 			if self.event.loadingLength:
-				scale = (self.event.loadingLength * self.project.viewScale) / float(len(self.event.levels))
+				duration = self.event.loadingLength
 			else:
-				scale = (self.event.duration * self.project.viewScale) / float(len(self.event.levels))
+				duration = self.event.duration
 			
-			# Draw waveform
-			x_pos = int(rect.x/scale)
-			x = 0
-			skipFactor = max(int(self._MIN_POINT_SEPARATION / scale), 1)
 			context.move_to(0,rect.height)
-					
-			# get levels list
-			fadedLevels = self.event.GetFadeLevels()
 			
-			for peak in fadedLevels[x_pos::skipFactor]:
-				x = (x_pos * scale) - rect.x
-				peakOnScreen = int(peak * rect.height)
+			levels = self.event.GetFadeLevels()
+			length = len(levels)
+			
+			# time offset of the start of the drawing area in milliseconds
+			starting_time = int(rect.x / self.project.viewScale * 1000)
+			starting_index = levels.find_endtime_index(starting_time)
+
+			x = 0
+			last_x = -2
+			skip_list = []
+			iterator = itertools.islice(levels, starting_index, length)
+			for endtime, peak in iterator:
+				x = int((endtime - starting_time) * self.project.viewScale / 1000)
+				
+				peakOnScreen = int(peak * rect.height / sys.maxint)
+				skip_list.append(peakOnScreen)
+				if (x - last_x) < self._MIN_POINT_SEPARATION:
+					continue
+				
+				peakOnScreen = sum(skip_list) / len(skip_list)
 				context.line_to(x, rect.height - peakOnScreen)
 				
+				skip_list = []
+				last_x = x
 				if x > rect.width:
 					break
-				x_pos += skipFactor
 			
 			context.line_to(x, rect.height)
 			
@@ -395,7 +436,7 @@ class EventViewer(gtk.DrawingArea):
 			
 			if self.event.isLoading:
 				# Write "Loading..." or "Downloading..."
-				if self.event.duration == 0:
+				if self.event.duration <= 0:
 					# for some file types gstreamer doesn't give us a duration
 					# so don't display the percentage
 					if self.event.isDownloading:
@@ -423,8 +464,6 @@ class EventViewer(gtk.DrawingArea):
 				#Draw event name
 				context.show_text(self.event.name)
 		
-		#set area to record where the cached surface goes
-		self.cachedDrawArea = rect
 		self.redrawWaveform = False
 
 	#_____________________________________________________________________
@@ -433,13 +472,19 @@ class EventViewer(gtk.DrawingArea):
 		"""
 		Called when the EventViewer gets destroyed.
 		It also destroys any child widget and disconnects itself from any
-		listening objects via Monitored.
+		gobject signals.
 		"""
-		self.project.RemoveListener(self)
-		self.event.RemoveListener(self)
+		self.project.disconnect_by_func(self.OnProjectZoom)
+		self.event.disconnect_by_func(self.OnEventSelected)
+		self.event.disconnect_by_func(self.OnEventCorrupt)
+		self.event.disconnect_by_func(self.OnEventLength)
+		self.event.disconnect_by_func(self.OnEventLoading)
+		self.event.disconnect_by_func(self.OnEventPosition)
+		self.event.disconnect_by_func(self.OnEventWaveform)
 		
 		#delete the cached images
-		del self.source
+		del self.sourceSmall
+		del self.sourceLarge
 		del self.cancelImg
 		self.destroy()
 	
@@ -495,12 +540,12 @@ class EventViewer(gtk.DrawingArea):
 			
 			if self.event.MayPlace(time):
 				self.event.start = time
-				self.lane.Update(self)
+				self.lane.UpdatePosition(self)
 				self.mouseAnchor = [x, y]
 			else:
 				temp = self.event.start
 				self.event.MoveButDoNotOverlap(time)
-				self.lane.Update(self)
+				self.lane.UpdatePosition(self)
 				
 				#MoveButDoNotOverlap() moves the event out of sync with the mouse
 				#and the mouseAnchor must be updated manually.
@@ -520,12 +565,6 @@ class EventViewer(gtk.DrawingArea):
 				selection_direction = "rtol"
 				self.fadeMarkers.reverse()
 			
-			if self.drawer.parent != self.lane.fixed:
-				#the drawer is not in the lane, we must add it
-				self.lane.fixed.put(self.drawer,1,75)
-				#update the lane so the drawer is shown
-				self.lane.Update()
-			
 			#set the drawer align position
 			self.drawerAlignToLeft = (selection_direction == "rtol")
 			#move the drawer to its proper position
@@ -572,6 +611,11 @@ class EventViewer(gtk.DrawingArea):
 			self.project.SelectInstrument(None)
 		self.event.SetSelected(True)
 		
+		#Don't allow editing while playing back.
+		#It must be here to avoid afecting the selection behavior
+		if self.mainview.isPlaying or self.mainview.isPaused:
+			return True
+		
 		# RMB: context menu
 		if mouse.button == 3:
 			self.ContextMenu(mouse)
@@ -814,7 +858,7 @@ class EventViewer(gtk.DrawingArea):
 		if self.event.selection == [0, 0]:
 			self.HideDrawer()
 
-		self.lane.Update(self)
+		self.lane.UpdatePosition(self)
 			
 		return True
 
@@ -918,7 +962,7 @@ class EventViewer(gtk.DrawingArea):
 			if self.isDragging:		
 				self.isDragging = False
 				if (self.eventStart != self.event.start):
-					self.event.Move(self.eventStart, self.event.start)
+					self.event.Move(self.event.start, self.eventStart)
 					return False #need to pass this button release up to RecordingView
 			elif self.isDraggingFade:
 				self.isDraggingFade = False
@@ -939,18 +983,12 @@ class EventViewer(gtk.DrawingArea):
 		if selection[0] > selection[1]:
 			self.event.selection = [selection[1], selection[0]]
 			selection_direction = "rtol"
-				
-		if self.drawer.parent != self.lane.fixed:
-			#the drawer is not in the lane, we must add it
-			self.lane.fixed.put(self.drawer,1,75)
-			
-			#set the drawer align position
-			self.drawerAlignToLeft = (selection_direction == "rtol")
-			#move the drawer to its proper position
-			self.UpdateDrawerPosition()
-			#update the lane so the drawer is shown
-			self.lane.Update()
-
+		
+		#set the drawer align position
+		self.drawerAlignToLeft = (selection_direction == "rtol")
+		#move the drawer to its proper position
+		self.UpdateDrawerPosition()
+		
 	#_____________________________________________________________________
 		
 	def OnMouseLeave(self, widget, event):
@@ -962,7 +1000,7 @@ class EventViewer(gtk.DrawingArea):
 			widget -- reserved for GTK callbacks, don't use it explicitly.
 			mouse -- GTK mouse event that fired this method call.
 		"""
-		if self.messageID:   #clesr status bar if not already clear
+		if self.messageID:   #clear status bar if not already clear
 			self.mainview.ClearStatusBar(self.messageID)
 			self.messageID = None
 		self.highlightCursor = None
@@ -978,13 +1016,18 @@ class EventViewer(gtk.DrawingArea):
 			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
 			position -- The position in the event to split
 		"""
-		if pos == 0.0:
-			return
-			
-		pos /= float(self.project.viewScale)
-		self.event.SplitEvent(pos)
-
-		self.lane.Update()
+		if self.event.selection != [0,0]:
+			undoAction = self.project.NewAtomicUndoAction()
+			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
+			self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
+			self.event.selection = [0,0]
+			self.HideDrawer()
+		else:
+			if pos == 0.0:
+				return
+			else:
+				pos /= float(self.project.viewScale)
+				self.event.SplitEvent(pos)
 		
 	#_____________________________________________________________________
 	
@@ -995,8 +1038,17 @@ class EventViewer(gtk.DrawingArea):
 		Parameters:
 			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		self.project.clipboardList = [self.event]
-		self.OnDelete()
+		if self.event.selection != [0,0]:
+			undoAction = self.project.NewAtomicUndoAction()
+			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
+			e = self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
+			self.project.clipboardList = [e]
+			e.Delete(_undoAction_=undoAction)
+			self.event.selection = [0,0]
+			self.HideDrawer()
+		else:
+			self.project.clipboardList = [self.event]
+			self.OnDelete()
 	
 	#_____________________________________________________________________
 	
@@ -1007,7 +1059,12 @@ class EventViewer(gtk.DrawingArea):
 		Parameters:
 			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		self.project.clipboardList = [self.event]
+		if self.event.selection != [0,0]:
+			e = self.event.CopySelection()
+			self.project.clipboardList = [e]
+			#We shouldn't hide the drawer here, unfriendly behaviour
+		else:
+			self.project.clipboardList = [self.event]
 	
 	#_____________________________________________________________________
 
@@ -1019,10 +1076,15 @@ class EventViewer(gtk.DrawingArea):
 		Parameters:
 			event -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		# delete event
-		self.event.Delete()
-		self.HideDrawer()
-		self.lane.Update()
+		if self.event.selection != [0,0]:
+			undoAction = self.project.NewAtomicUndoAction()
+			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
+			e = self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
+			e.Delete(_undoAction_=undoAction)
+			self.event.selection = [0,0]
+			self.HideDrawer()
+		else:
+			self.event.Delete()
 	
 	#_____________________________________________________________________
 		
@@ -1048,9 +1110,10 @@ class EventViewer(gtk.DrawingArea):
 		"""
 		Hide the drawer.
 		"""
-		if self.drawer.parent == self.lane.fixed:
-			self.lane.fixed.remove(self.drawer)
+		
+		self.lane.RemoveDrawer(self.drawer)
 
+	#_____________________________________________________________________
 	
 	def do_size_request(self, requisition):
 		"""
@@ -1069,65 +1132,95 @@ class EventViewer(gtk.DrawingArea):
 		if not (self.small):
 			requisition.height = 77
 		else:
-			rect = self.get_allocation()
-			
-			if rect.height < 30:
-				requisition.height = 30
-			else:
-				requisition.height = rect.height
+			requisition.height = 30
+#			rect = self.get_allocation()
+#			
+#			if rect.height < 30:
+#				requisition.height = 30
+#			else:
+#				requisition.height = rect.height
 
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None, *extra):
+	def OnEventPosition(self, event):
 		"""
-		Called when a change of state is signalled by any of the
-		objects this view is 'listening' to.
-		Redraws the Event if any of its audio has been modified.
-		
-		Parameters:
-			obj -- object changing state.
-			change -- the change which has occured.
-			extra -- extra parameters passed by the caller.
+		Callback function for when the position of the event changes.
 		"""
-		if change == self.event.WAVEFORM:
-			self.redrawWaveform = True
-			self.UpdateFadeMarkers()
+		self.SetAccessibleName()
+		self.lane.UpdatePosition(self)
+	
+	#_____________________________________________________________________
+	
+	def OnEventWaveform(self, event):
+		"""
+		Callback function for when the waveform of the event changes.
+		"""
+		self.redrawWaveform = True
+		self.UpdateFadeMarkers()
+		self.queue_draw()
 		
-		elif change == self.event.MOVE:
-			self.lane.Update(self)
+	#_____________________________________________________________________
+	
+	def OnEventLoading(self, event):
+		"""
+		Callback function for when the loading status of the event changes.
+		"""
+		self.drawer.set_sensitive(not self.event.isLoading)
+		self.queue_draw()
 		
-		elif change == self.event.LENGTH:
-			self.redrawWaveform = True
-			self.queue_resize()
-			
-		elif change == self.event.LOADING:
-			self.drawer.set_sensitive(not self.event.isLoading)
+	#_____________________________________________________________________
+	
+	def OnEventLength(self, event):
+		"""
+		Callback function for when the length of the event changes.
+		"""
+		self.redrawWaveform = True
+		self.SetAccessibleName()
+		self.queue_resize()
+		self.queue_draw()
 		
-		elif change == self.event.CORRUPT and self.small==False:
-			message="%s %s\n\n%s" % (_("Error loading file:"), self.event.filelabel, 
+	#_____________________________________________________________________
+	
+	def OnEventSelected(self, event):
+		"""
+		Callback function for when the event is selected or de-selected.
+		"""
+		self.queue_draw()
+		
+	#_____________________________________________________________________
+	
+	def OnEventCorrupt(self, event, error):
+		"""
+		Callback function for when the event's file is found to be corrupt.
+		"""
+		message="%s %s\n\n%s" % (_("Error loading file:"), self.event.filelabel, 
 					_("Please make sure the file exists, and the appropriate plugin is installed."))
 			
-			outputtext = "\n\n".join(extra)
-			outputtext = "\n\n".join((message, outputtext))
-			
-			dlg = gtk.MessageDialog(None,
-				gtk.DIALOG_MODAL,
-				gtk.MESSAGE_ERROR,
-				gtk.BUTTONS_CLOSE,
-				outputtext)
-			dlg.connect('response', lambda dlg, response: dlg.destroy())
-			dlg.show()
-			self.OnDelete()
+		outputtext = "\n\n".join((message, error))
+		
+		dlg = gtk.MessageDialog(None,
+			gtk.DIALOG_MODAL,
+			gtk.MESSAGE_ERROR,
+			gtk.BUTTONS_CLOSE,
+			outputtext)
+		dlg.connect('response', lambda dlg, response: dlg.destroy())
+		dlg.show()
+		self.OnDelete()
+		
+	#_____________________________________________________________________
 	
-		elif obj.__class__ == Project and self.currentScale != self.project.viewScale:
+	def OnProjectZoom(self, project):
+		"""
+		Callback for when the zoom level of the project changes.
+		
+		Parameters:
+			project -- The project instance that send the signal.
+		"""
+		if self.currentScale != self.project.viewScale:
 			self.redrawWaveform = True
 			self.queue_resize()
-			self.last_num_levels = len(self.event.levels)
 			self.currentScale = self.project.viewScale
-
-		self.SetAccessibleName()
-		
-		self.queue_draw()
+			self.queue_draw()
 
 	#_____________________________________________________________________
 
@@ -1222,9 +1315,6 @@ class EventViewer(gtk.DrawingArea):
 			reverseSelectionPoints -- True if the selection points should
 										be reversed.
 		"""
-		if self.drawer.parent != self.lane.fixed:
-			#drawer is not in lane
-			return
 		
 		if reverseSelectionPoints:
 			selection = [self.event.selection[1], self.event.selection[0]]
@@ -1246,7 +1336,7 @@ class EventViewer(gtk.DrawingArea):
 				width = 40 # fudge it because it has no width initially
 			x = int(self.PixXFromSec(selection[1]) - width)
 		
-		self.lane.fixed.move(self.drawer,eventx + x,75)
+		self.lane.PutDrawer(self.drawer, eventx + x)
 		#don't update the lane because it calls us and that might cause infinite loop
 
 	#_____________________________________________________________________
@@ -1327,4 +1417,18 @@ class EventViewer(gtk.DrawingArea):
 		
 	#_____________________________________________________________________
 
+	def ChangeSize(self, small):
+		"""
+		Changes size of event viewer.
+		
+		Parameters:
+			small -- True if the event viewer is to change to small
+		"""
+		self.small = small
+		self.queue_resize()
+		self.queue_draw()
+			
+	#____________________________________________________________________	
+
+
 #=========================================================================
