@@ -325,7 +325,6 @@ class ExtensionAPI:
 					#instr[0] is the default Instrument name
 					#instr[1] is the Instrument type
 					#instr[2] is the Instrument icon in the .instr file
-				self.mainapp.UpdateDisplay()
 				return newInstr.id
 		return -1
 	
@@ -354,7 +353,6 @@ class ExtensionAPI:
 			instrumentID -- ID of the Instrument to be removed.
 		"""
 		self.mainapp.project.DeleteInstrument(instrumentID)
-		self.mainapp.UpdateDisplay()
 		
 		#time for a Newfie Joke: 
 		#How many Newfies does it take to go ice fishing?
@@ -771,7 +769,7 @@ class ExtensionAPI:
 		Parameters:
 			description -- string for the drop down box. i.e: 'Ogg Vorbis (.ogg)'.
 			extension -- string of the file extension without a '.'. i.e: 'ogg'.
-			encodeBin -- string used by gst_parse_bin_from_description to create
+			encodeBin -- string used by gst.parse_bin_from_description to create
 						a bin that can encode and mux the audio when added to a
 						pipeline. i.e: 'vorbisenc ! oggmux'.
 			checkIfValid -- If True, Jokosher will check if the encodeBin string is valid before adding
@@ -788,7 +786,7 @@ class ExtensionAPI:
 			
 		if checkIfValid:
 			try:
-				bin = gst.gst_parse_bin_from_description("audioconvert ! %s" % encodeBin, True)
+				bin = gst.parse_bin_from_description("audioconvert ! %s" % encodeBin, True)
 				del bin
 			except gobject.GError:
 				return 3
@@ -811,7 +809,7 @@ class ExtensionAPI:
 		Parameters:
 			description -- string for the drop down box. i.e: 'Ogg Vorbis (.ogg)'.
 			extension -- string of the file extension without a '.'. i.e: 'ogg'.
-			encodeBin -- string used by gst_parse_bin_from_description to create
+			encodeBin -- string used by gst.parse_bin_from_description to create
 						a bin that can encode and mux the audio when added to a
 						pipeline. i.e: 'vorbisenc ! oggmux'.
 		Returns:
@@ -916,7 +914,6 @@ class ExtensionAPI:
 			bpm -- the beats per minute to set.
 		"""
 		self.mainapp.project.SetBPM(float(bpm))
-		self.mainapp.project.PrepareClick()
 		
 	#_____________________________________________________________________
 	
