@@ -20,7 +20,6 @@ import urlparse # To split up URI's
 import gobject
 import Event
 import UndoSystem
-from Monitored import Monitored
 import Utils
 
 import Globals
@@ -30,7 +29,7 @@ _ = gettext.gettext
 
 #=========================================================================	
 
-class Instrument(Monitored):
+class Instrument(gobject.GObject):
 	"""
 	This module is the non-gui class the represents Instruments. Instruments
 	represent a track of audio that can contain many different sources in sequence.
@@ -48,6 +47,39 @@ class Instrument(Monitored):
 	"""
 	LADSPA_ELEMENT_CAPS = "audio/x-raw-float, width=(int)32, rate=(int)[ 1, 2147483647 ], channels=(int)1, endianness=(int)1234"
 	
+	"""
+	Signals:
+		"arm" -- This instrument has been armed or dis-armed for recording.
+		"effect" -- The effects for this instrument have changed. See below:
+			"effect::added" -- An effect was added to this instrument.
+			"effect::removed" -- An effect was removed from this instrument.
+			"effect::reordered" -- An effect on this instrument has changed its position.
+		"event" -- The events for this instrument have changed. The event ID will be passed as a parameter. See below:
+			"event::added" -- An event was added to this instrument.
+			"event::removed" -- An event was removed from this instrument.
+		"image" -- The image for this instrument has changed.
+		"mute" -- This instrument has been muted or unmuted.
+		"name" -- The name of this instrument has changed.
+		"recording-done" -- The event recording on this instrument has finished recording.
+		"selected" -- This instrument has been selected or deselected.
+		"visible" -- This instrument has been maximised or minimized in the mix view.
+		"volume" -- The volume value for this instrument has changed.
+	"""
+	
+	__gsignals__ = {
+		"arm"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"effect"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, () ),
+		"event"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,) ),
+		"image"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"mute"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"name"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"recording-done"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"selected"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"solo"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"visible"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"volume"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () )
+	}
+	
 	def __init__(self, project, name, type, pixbuf, id=None):
 		"""
 		Creates a new instance of Instrument.
@@ -59,11 +91,10 @@ class Instrument(Monitored):
 			pixbuf -- image of the Instrument resource object.
 			id -- unique ID value for the Instrument.
 		"""
-		Monitored.__init__(self)
+		gobject.GObject.__init__(self)
 		
 		self.project = project
 		
-		self.path = ""				# The 'audio' directory for this instrument
 		self.events = []				# List of events attached to this instrument
 		self.graveyard = []			# List of events that have been deleted (kept for undo)
 		self.effects = []				# List of GStreamer effect elements
@@ -316,7 +347,7 @@ class Instrument(Monitored):
 		#give it a lambda for a callback that does nothing, so we don't have to wait
 		startSrcPad.set_blocked_async(False, lambda x,y: False)
 		
-		self.StateChanged("effects")
+		self.emit("effect::added")
 		
 		return effectElement
 	
@@ -363,7 +394,7 @@ class Instrument(Monitored):
 		#give it a lambda for a callback that does nothing, so we don't have to wait
 		startSrcPad.set_blocked_async(False, lambda x,y: False)
 		
-		self.StateChanged("effects")
+		self.emit("effect::removed")
 	
 	#_____________________________________________________________________
 	
@@ -447,13 +478,13 @@ class Instrument(Monitored):
 		#give it a lambda for a callback that does nothing, so we don't have to wait
 		startSrcPad.set_blocked_async(False, lambda x,y: False)
 		
-		self.StateChanged("effects")
+		self.emit("effect:reordered")
 	
 	#_____________________________________________________________________
 	
 	def GetRecordingEvent(self):
 		"""
-		Obtain an Event suitable for recording. *CHECK*
+		Obtain an Event suitable for recording.
 		Returns:
 			an Event suitable for recording.
 		"""
@@ -461,9 +492,15 @@ class Instrument(Monitored):
 		event.start = self.project.transport.GetPosition()
 		event.isRecording = True
 		event.name = _("Recorded audio")
-		event.file = "%s_%d_%d.ogg" % (os.path.join(self.path, Globals.FAT32SafeFilename(self.name)), self.id, int(time.time()))
+		
+		ext = Globals.settings.recording["file_extension"]
+		filename = "%s_%d.%s" % (Globals.FAT32SafeFilename(self.name), event.id, ext)
+		event.file = os.path.join(self.project.audio_path, filename)
+		event.levels_file = filename + Event.Event.LEVELS_FILE_EXTENSION
+		
 		#must add it to the instrument's list so that an update of the event lane will not remove the widget
 		self.events.append(event)
+		self.emit("event::added", event)
 		return event
 
 	#_____________________________________________________________________
@@ -480,7 +517,7 @@ class Instrument(Monitored):
 		event.isRecording = False
 		event.GenerateWaveform()
 		self.temp = event.id
-		self.StateChanged()
+		self.emit("recording-done")
 	
 	#_____________________________________________________________________
 	
@@ -547,16 +584,17 @@ class Instrument(Monitored):
 			the added Event.
 		"""
 		filelabel=file
+		event_id = self.project.GenerateUniqueID(None,  reserve=False)
+		name = os.path.basename(file)
+		root,  extension = os.path.splitext(name.replace(" ", "_"))
+		
+		if extension:
+			newfile = "%s_%d.%s" % (root, event_id, extension)
+		else:
+			newfile = "%s_%d" % (root, event_id)
 
 		if copyfile:
-			basename = os.path.split(file.replace(" ", "_"))[1]
-			basecomp = basename.rsplit('.', 1)
-			if len(basecomp) > 1:
-				newfile = "%s_%d_%d.%s" % (basecomp[0], self.id, int(time.time()), basecomp[len(basecomp)-1])
-			else:
-				newfile = "%s_%d_%d" % (basecomp[0], self.id, int(time.time()))
-
-			audio_file = os.path.join(self.path, newfile)
+			audio_file = os.path.join(self.project.audio_path, newfile)
 			
 			try:
 				shutil.copyfile(file, audio_file)
@@ -566,11 +604,8 @@ class Instrument(Monitored):
 			self.project.deleteOnCloseAudioFiles.append(audio_file)
 			
 			file = audio_file
-			name = basename
-		else:
-			name = file.split(os.sep)[-1]
 
-		ev = Event.Event(self, file, None, filelabel)
+		ev = Event.Event(self, file, event_id, filelabel)
 		ev.start = start
 		ev.name = name
 		self.events.append(ev)
@@ -578,7 +613,7 @@ class Instrument(Monitored):
 
 		self.temp = ev.id
 		
-		self.StateChanged()
+		self.emit("event::added", ev)
 		
 		return ev
 		
@@ -600,14 +635,15 @@ class Instrument(Monitored):
 		Returns:
 			the added Event.
 		"""
+		event_id = self.project.GenerateUniqueID(None,  reserve=False)
 		# no way of knowing whether there's a filename, so make one up
-		newfile = "%d_%d" % (self.id, int(time.time()))
-
-		audio_file = os.path.join(self.path, newfile)
+		newfile = str(event_id)
+		
+		audio_file = os.path.join(self.project.audio_path, newfile)
 		self.project.deleteOnCloseAudioFiles.append(audio_file)
 		
 		# Create the event now so we can return it, and fill in the file later
-		ev = Event.Event(self, audio_file, None, url)
+		ev = Event.Event(self, audio_file, event_id, url)
 		ev.start = start
 		ev.name = os.path.split(audio_file)[1]
 		ev.isDownloading = True
@@ -621,7 +657,7 @@ class Instrument(Monitored):
 			raise UndoSystem.CancelUndoCommand()
 		
 		self.temp = ev.id
-		self.StateChanged()
+		self.emit("event::added", ev)
 		
 		return ev
 	
@@ -641,7 +677,7 @@ class Instrument(Monitored):
 		ev.start = start
 		for i in ["duration", "name", "offset"]:
 			setattr(ev, i, getattr(event, i))
-		ev.levels = event.levels[:]
+		ev.levels_list = event.levels_list.copy()
 		ev._Event__fadePointsDict = event._Event__fadePointsDict.copy()
 		ev._Event__UpdateAudioFadePoints()
 		
@@ -650,7 +686,7 @@ class Instrument(Monitored):
 		ev.MoveButDoNotOverlap(ev.start)
 		
 		self.temp = ev.id
-		self.StateChanged()
+		self.emit("event::added", ev)
 	
 	#_____________________________________________________________________
 	
@@ -666,10 +702,11 @@ class Instrument(Monitored):
 		
 		self.graveyard.append(event)
 		self.events.remove(event)
-		self.composition.remove(event.filesrc)
+		event.DestroyFilesource()
 		event.StopGenerateWaveform(False)
 		
 		self.temp = eventid
+		self.emit("event::removed", event)
 	
 	#_____________________________________________________________________
 	
@@ -686,10 +723,11 @@ class Instrument(Monitored):
 		self.events.append(event)
 		self.graveyard.remove(event)
 		event.CreateFilesource()
-		if event.isLoading:
+		if event.isLoading or not event.levels_list:
 			event.GenerateWaveform()
 		
 		self.temp = eventid
+		self.emit("event::added", event)
 	
 	#_____________________________________________________________________
 
@@ -734,7 +772,7 @@ class Instrument(Monitored):
 			self.volume = volume
 			self.UpdateVolume()
 			self.project.unsavedChanges = True
-			self.StateChanged("volume")
+			self.emit("volume")
 
 	#_____________________________________________________________________
 	
@@ -759,7 +797,7 @@ class Instrument(Monitored):
 		if self.name != name:
 			self.temp = self.name
 			self.name = name
-			self.StateChanged()
+			self.emit("name")
 	
 	#_____________________________________________________________________
 	
@@ -769,7 +807,7 @@ class Instrument(Monitored):
 		Arms/Disarms the Instrument for recording.
 		"""
 		self.isArmed = not self.isArmed
-		self.StateChanged()
+		self.emit("arm")
 		
 	#_____________________________________________________________________
 	
@@ -781,6 +819,10 @@ class Instrument(Monitored):
 		Parameters:
 			wasSolo --	True = the Instrument had solo mode enabled.
 						False = the Instrument was not in solo mode.
+						
+		Considerations:
+			The signal "mute" is not emitted here because it is emitted in
+			the OnMute() function.
 		"""
 		self.temp = self.isSolo
 		self.isMuted = not self.isMuted
@@ -794,8 +836,6 @@ class Instrument(Monitored):
 			self.project.OnAllInstrumentsMute()
 		else:
 			self.OnMute()
-			
-		self.StateChanged()
 	
 	#_____________________________________________________________________
 	
@@ -819,7 +859,7 @@ class Instrument(Monitored):
 			self.project.soloInstrCount += 1
 		
 		self.project.OnAllInstrumentsMute()
-		self.StateChanged()
+		self.emit("solo")
 	
 	#_____________________________________________________________________
 	
@@ -836,7 +876,7 @@ class Instrument(Monitored):
 		if self.isVisible != visible:
 			self.temp = self.isVisible
 			self.isVisible = visible
-			self.StateChanged()
+			self.emit("visible")
 	
 	#_____________________________________________________________________
 	
@@ -848,10 +888,10 @@ class Instrument(Monitored):
 			sel -- 	True = the Instrument is currently selected.
 					False = the Instrument is not currently selected.
 		"""
-		# No need to call StateChanged when there is no change in selection state
+		# No need to emit signal when there is no change in selection state
 		if self.isSelected is not sel:
 			self.isSelected = sel
-			self.StateChanged()
+			self.emit("selected")
 	
 	#_____________________________________________________________________
 	
@@ -864,6 +904,8 @@ class Instrument(Monitored):
 			self.volumeElement.set_property("mute", True)
 		else:
 			self.volumeElement.set_property("mute", False)
+		
+		self.emit("mute")
 	
 	#_____________________________________________________________________
 	
@@ -949,17 +991,20 @@ class Instrument(Monitored):
 		if type != self.instrType and pixbufList:
 			pixbuf = pixbufList[0]
 		else:
-			return
+			raise UndoSystem.CancelUndoCommand()
 
-		oldname = self.name.lower().replace(" ","")
-		#if the instrument name has not been modified by the user, we can replace it.
-		if oldname == self.instrType:
-			self.name = name
+		for tuple_ in Globals.getCachedInstruments():
+			if tuple_[1] == self.instrType:
+				if tuple_[0] == self.name:
+					#if the instrument name has not been modified by the user, we can replace it.
+					self.name = name
+					self.emit("name")
+				break
 
 		self.instrType = type
 		self.pixbuf = pixbuf
 
-		self.StateChanged("image")
+		self.emit("image")
 
 	#_____________________________________________________________________
 
