@@ -12,8 +12,8 @@ import gtk
 import pango
 from EventLaneViewer import *
 import Globals
-import InstrumentEffectsDialog
 import AddInstrumentDialog
+import ControlsBox
 import Utils
 import gettext
 _ = gettext.gettext
@@ -62,13 +62,13 @@ class InstrumentViewer(gtk.EventBox):
 		self.small = small
 		self.projectview = projectview
 		self.mainview = mainview
-		self.instrument.AddListener(self)
-		self.effectsDialog = None		#the instrument effects dialog (to make sure more than one is never opened)
 		
 		self.Updating = False
 		
 		#get the default colour for the current theme
-		self.UNSELECTED_COLOR = self.get_style().bg[0]
+		self.UNSELECTED_COLOR = self.rc_get_style().bg[gtk.STATE_NORMAL]
+		#use base instead of bg colours so that we get the lighter colour that is used for list items in TreeView.
+		self.SELECTED_COLOUR = self.rc_get_style().base[gtk.STATE_SELECTED]
 		
 		self.mainBox = gtk.HBox()
 		self.add(self.mainBox)
@@ -120,56 +120,11 @@ class InstrumentViewer(gtk.EventBox):
 		self.labelbox.pack_start(self.image, False)
 		self.labelbox.pack_end(self.labeleventbox)
 		self.headerBox.pack_start(self.labelbox)
-		self.controlsBox = gtk.HBox()
+		self.controlsBox = ControlsBox.ControlsBox(project,mainview,instrument,includeEffects=True)
 		self.headerBox.pack_start(self.controlsBox, False)
 		
-		# define the tooltip messages and images for buttons that change states
-		self.recTipDisabled = _("Enable this instrument for recording")
-		self.recTipEnabled = _("Disable this instrument for recording")
-		self.muteTipDisabled = _("Mute - silence this instrument")
-		self.muteTipEnabled = _("Unmute - hear this instrument")
-		self.soloTipDisabled = _("Activate Solo - silence all other instruments")
-		self.soloTipEnabled = _("Deactivate Solo - hear all the instruments")
-		
-		self.recImgDisabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_arm.png"))
-		self.recImgEnabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_disarm.png"))
-		self.soloImgDisabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_solo.png"))
-		self.soloImgEnabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_group.png"))
-		self.muteImgDisabled = Utils.GetIconThatMayBeMissing("stock_volume", gtk.ICON_SIZE_BUTTON, False)
-		self.muteImgEnabled = Utils.GetIconThatMayBeMissing("stock_volume-mute", gtk.ICON_SIZE_BUTTON, False)
-		
-		if not (self.small):
-			self.recTip = gtk.Tooltips()
-			self.recButton = gtk.ToggleButton("")
-			self.recTip.set_tip(self.recButton, self.recTipEnabled, None)
-			self.recButton.connect("toggled", self.OnArm)
-			
-			self.muteButton = gtk.ToggleButton("")
-			self.muteButton.connect("toggled", self.OnMute)
-			self.muteTip = gtk.Tooltips()
-			self.muteTip.set_tip(self.muteButton, self.muteTipDisabled, None)
-			
-			self.soloButton = gtk.ToggleButton("")
-			self.soloTip = gtk.Tooltips()
-			self.soloTip.set_tip(self.soloButton, self.soloTipDisabled, None)
-			self.soloButton.connect("toggled", self.OnSolo)
-			
-			self.propsButton = gtk.Button()
-			procimg = gtk.Image()
-			procimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_effectsapply.png"))
-			self.propsButton.set_image(procimg)
-
-			self.propsButton.connect("button_press_event", self.OnInstrumentEffects)
-			self.propsTip = gtk.Tooltips()
-			self.propsTip.set_tip(self.propsButton, _("Instrument Effects"), None)
-			
-			self.controlsBox.add(self.recButton)
-			self.controlsBox.add(self.muteButton)
-			self.controlsBox.add(self.soloButton)
-			self.controlsBox.add(self.propsButton)
-		else:
-			self.separator = gtk.HSeparator()
-			self.headerBox.pack_end(self.separator, False, True)
+		self.separator = gtk.HSeparator()
+		self.headerBox.pack_end(self.separator, False, True)
 		self.instrument.isSelected = False
 		
 		# Begin Drag and Drop code
@@ -183,45 +138,19 @@ class InstrumentViewer(gtk.EventBox):
 		# Connect to drag_begin to add a custom icon
 		self.headerEventBox.connect('drag_begin', self.OnDragBegin)
 		self.headerEventBox.connect('drag_drop', self.OnDragDrop)
+		self.headerEventBox.connect('drag_end', self.OnDragEnd)
 		
-		self.Update()
+		self.instrument.connect("name", self.OnInstrumentName)
+		self.instrument.connect("image", self.OnInstrumentImage)
+		self.instrument.connect("selected", self.OnInstrumentSelected)
 
-	#_____________________________________________________________________
+		#set the appropriate colour if the instrument it already selected.
+		self.OnInstrumentSelected()
+		self.show_all()
+		self.labelbox.show()
+		if self.small:
+			self.controlsBox.hide()
 
-	def OnMute(self, widget):
-		"""
-		Toggles muting the instrument on/off.
-		It will also update the pressed in/out look of the button.
-		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-		"""
-		if not self.Updating:
-			self.instrument.ToggleMuted(wasSolo=False)
-	
-	#_____________________________________________________________________
-
-	def OnArm(self, widget):
-		"""
-		Toggles arming the instrument on/off.
-		It will also update the pressed in/out look of the button.
-		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-		"""
-		if not self.Updating:
-			self.instrument.ToggleArmed()
-		
-	#_____________________________________________________________________
-	
-	def OnSolo(self, widget):
-		"""
-		Toggles soloing the instrument on/off.
-		It will also update the pressed in/out look of the button.
-		"""
-		if not self.Updating:
-			self.instrument.ToggleSolo(False)
-		
 	#_____________________________________________________________________
 
 	def OnSelect(self, widget, event):
@@ -256,8 +185,8 @@ class InstrumentViewer(gtk.EventBox):
 		of the correspondent Instrument
 		
 		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-			event -- reserved for GTK callbacks, don't use it explicitly.
+			widget -- GTK callback.
+			event -- GTK callback.
 			
 		Returns:
 			True -- continue GTK signal propagation. *CHECK*
@@ -278,26 +207,45 @@ class InstrumentViewer(gtk.EventBox):
 			#set the entry to the same width as the label so that the header doesnt resize
 			self.editlabel.set_size_request(width, -1)
 			self.editlabel.connect("activate", self.OnAcceptEditLabel)
+			self.editlabel.connect("key_press_event", self.OnEditLabelKey)
 			self.editlabel.show()
 			
 			self.labelbox.pack_end(self.editlabel)
 			self.editlabel.grab_focus()
 			self.editlabelPacked = True
 			self.mainview.instrNameEntry = self.editlabel
+			
 			return True
 	
 	#_____________________________________________________________________
 
+	def OnEditLabelKey(self, widget, event):
+		"""
+		Handles the key presses while editing the instrument name label.
+		Used to make the escape key save the name and then return to normal mode.
+		
+		Parameters:
+			widget -- GTK callback.
+			event -- GTK callback.
+		"""
+		key = gtk.gdk.keyval_name(event.keyval)
+		
+		if key == "Escape":
+			self.editlabel.set_text("")
+			self.OnAcceptEditLabel()
+
+	#_____________________________________________________________________
+
 	def OnAcceptEditLabel(self, widget=None):
 		"""
 		Called after the instrument label has been edited.
 		This method updates the instrument label with the label the user entered.
 		
 		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
+			widget -- GTK callback.
 		"""
-		# change instrument name then replace edit label with normak label
-		if self.editlabelPacked:	
+		# change instrument name then replace edit label with normal label
+		if self.editlabelPacked:
 			name = self.editlabel.get_text()
 			if name != "":
 				self.instrlabel.set_text(name)
@@ -319,64 +267,20 @@ class InstrumentViewer(gtk.EventBox):
 		Called when the InstrumentViewer is closed
 		This method also destroys the corresponding EventLaneViewer.
 		"""
-		self.instrument.RemoveListener(self)
+		self.instrument.disconnect_by_func(self.OnInstrumentImage)
 		self.eventLane.Destroy()
 		self.destroy()
 	
 	#_____________________________________________________________________
 	
-	def Update(self):
+	def OnInstrumentSelected(self, instrument=None):
 		"""
-		Called when requested by projectview.Update() to update	the display,
-		in response to a change in state in any object is it listening to.
-		In turn calls EventLaneViewer.Update() for its EventLaneViewer.
-		"""
-		self.Updating = True
-
-		if not self.small:
-			self.recButton.set_active(self.instrument.isArmed)
-			self.recTip.enable()
-			self.muteButton.set_active(self.instrument.actuallyIsMuted)
-			self.soloButton.set_active(self.instrument.isSolo)
-			self.soloTip.enable()
-
-			# update the mute button image and tooltip
-			image = gtk.Image()
-			if self.instrument.actuallyIsMuted:
-				image.set_from_pixbuf(self.muteImgEnabled)
-				self.muteButton.set_image(image)
-				self.muteTip.set_tip(self.muteButton, self.muteTipEnabled, None)
-			else:
-				image.set_from_pixbuf(self.muteImgDisabled)
-				self.muteButton.set_image(image)
-				self.muteTip.set_tip(self.muteButton, self.muteTipDisabled, None)
-			
-			# update the arm button image and tooltip	
-			image = gtk.Image()
-			if self.instrument.isArmed:
-				image.set_from_pixbuf(self.recImgEnabled)
-				self.recButton.set_image(image)
-				self.recTip.set_tip(self.recButton, self.recTipEnabled, None)
-			else:
-				image.set_from_pixbuf(self.recImgDisabled)
-				self.recButton.set_image(image)
-				self.recTip.set_tip(self.recButton, self.recTipDisabled, None)
-				
-			# update the solo button image and tooltip
-			image = gtk.Image()
-			if self.instrument.isSolo:
-				image.set_from_pixbuf(self.soloImgEnabled)
-				self.soloButton.set_image(image)
-				self.soloTip.set_tip(self.soloButton, self.soloTipEnabled, None)
-			else:
-				image.set_from_pixbuf(self.soloImgDisabled)
-				self.soloButton.set_image(image)
-				self.soloTip.set_tip(self.soloButton, self.soloTipDisabled, None)
+		Callback for when the instrument's selected status changes.
 		
+		Parameters:
+			instrument -- the instrument instance that send the signal.
+		"""
 		if self.instrument.isSelected:
-			#For some reason, putting self.style.base[3] in __init__ makes it return the wrong colour.
-			self.SELECTED_COLOUR = self.get_style().base[3]
-			
 			self.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
 			self.headerEventBox.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
 			self.labeleventbox.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
@@ -387,15 +291,20 @@ class InstrumentViewer(gtk.EventBox):
 			self.headerEventBox.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 			self.labeleventbox.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 			self.eventLane.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
-
-		self.instrlabel.set_text(self.instrument.name)
-		if self.editlabelPacked:
-			self.OnAcceptEditLabel()
-		self.eventLane.Update()
-		self.Updating = False
-
+	
 	#______________________________________________________________________
 
+	def OnInstrumentName(self, instrument=None):
+		"""
+		Callback for when the instrument's name changes.
+		
+		Parameters:
+			instrument -- the instrument instance that send the signal.
+		"""
+		self.instrlabel.set_text(self.instrument.name)
+	
+	#_____________________________________________________________________
+	
 	def OnMouseMove(self, widget, event):
 		"""
 		Called when the mouse cursor enters or leaves the instrument name label area.
@@ -429,39 +338,9 @@ class InstrumentViewer(gtk.EventBox):
 		padding = width - self.headerBox.size_request()[0]
 		self.headerAlign.set_padding(0, 0, 0, padding)
 
-
 	#______________________________________________________________________
 
-	def OnInstrumentEffects(self, widget, mouse):
-		"""
-		Creates and shows the instrument effects dialog
-		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-			mouse -- reserved for GTK callbacks, don't use it explicitly.
-		"""
-		Globals.debug("props button pressed")
-		if not self.effectsDialog:
-			self.effectsDialog = InstrumentEffectsDialog.InstrumentEffectsDialog(
-					self.instrument,
-					self.OnInstrumentEffectsDestroyed,
-					self.mainview.icon)
-		else:
-			self.effectsDialog.BringWindowToFront()
 
-	#______________________________________________________________________
-	
-	def OnInstrumentEffectsDestroyed(self, window):
-		"""
-		Called when the InstrumentEffectsDialog is destroyed.
-		
-		Parameters:
-			window -- reserved for GTK callbacks, don't use it explicitly.
-		"""
-		self.effectsDialog = None
-		
-	#______________________________________________________________________
-	
 	def OnDragMotion(self, widget, context, x, y, time):
 		"""
 		Called each time the user moves the mouse while dragging.
@@ -513,7 +392,36 @@ class InstrumentViewer(gtk.EventBox):
 	def OnDragDrop(self, widget, context, x, y, time):
 		"""
 		Called when the user releases MOUSE1, finishing a drag and drop
-		procedure.
+		procedure. This callback will only be called if the drag is dropped
+		on a widget that can handle the drop event. Otherwise only the
+		"drag_end" signal will be emitted, This is why the MoveInstrument()
+		function is called in OnDragEnd().
+			
+		Parameters:
+			widget -- InstrumentViewer being dragged.
+			context -- reserved for GTK callbacks, don't use it explicitly.
+			x -- reserved for GTK callbacks, don't use it explicitly.
+			y -- reserved for GTK callbacks, don't use it explicitly.
+			time -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		box = self.GetInstrumentViewVBox()
+		position = box.get_children().index(self)
+		
+		if self.project.instruments.index(self.instrument) == position:
+			#there is no change in position
+			context.finish(False, False, time)
+		else:
+			context.finish(True, False, time)
+	
+	#______________________________________________________________________
+	
+	def OnDragEnd(self, widget, context):
+		"""
+		Called when the user releases MOUSE1, finishing a drag and drop
+		procedure. This callback will only be called if the drag is dropped
+		on a widget that can handle the drop event. Otherwise only the
+		"drag_end" signal will be emitted, This is why the MoveInstrument()
+		function is called in OnDragEnd().
 		Calls MoveInstrument, which moves the dragged instrument to
 		the end position in the self.project.instruments array.
 		
@@ -527,17 +435,15 @@ class InstrumentViewer(gtk.EventBox):
 			y -- reserved for GTK callbacks, don't use it explicitly.
 			time -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		id = self.instrument.id
+		
 		box = self.GetInstrumentViewVBox()
 		position = box.get_children().index(self)
 		
-		if self.project.instruments.index(self.instrument) == position:
-			#there is no change in position
-			return
+		if self.project.instruments.index(self.instrument) != position:
+			#there was a change in position
+			id = self.instrument.id
+			self.project.MoveInstrument(id, position)
 		
-		self.project.MoveInstrument(id, position)
-		context.finish(True, False, time)
-	
 	#______________________________________________________________________
 
 	def GetInstrumentViewVBox(self):
@@ -555,20 +461,16 @@ class InstrumentViewer(gtk.EventBox):
 	
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None, *extra):
+	def OnInstrumentImage(self, instrument):
 		"""
-		Called when a change of state is signalled by any of the
-		objects this view is 'listening' to.
+		Callback for when the instrument's image changes.
 		
 		Parameters:
-			obj -- object changing state. *CHECK*
-			change -- the change which has occured.
-			extra -- extra parameters passed by the caller.
+			instrument -- the instrument instance that send the signal.
 		"""
-		if change == "image":
-			self.image.clear()
-			self.image.set_from_pixbuf(self.instrument.pixbuf)
-	
+		self.image.clear()
+		self.image.set_from_pixbuf(self.instrument.pixbuf)
+		
 	#______________________________________________________________________
 
 	def OnChangeInstrumentType(self, widget, event):
@@ -594,5 +496,24 @@ class InstrumentViewer(gtk.EventBox):
 		    
 	#______________________________________________________________________
 
+	def ChangeSize(self, small):
+		"""
+		Changes the size of the instrument viewer
+		
+		Parameters:
+			small -- True if the instrument viewer is to be small.
+		"""
+		self.small = small
+		self.eventLane.ChangeSize(small)
+		if self.small:
+			pb = self.instrument.pixbuf.scale_simple(20, 20, gtk.gdk.INTERP_BILINEAR)
+			self.image.set_from_pixbuf(pb)
+			self.controlsBox.hide()
+			self.separator.show()
+		else:
+			self.image.set_from_pixbuf(self.instrument.pixbuf)
+			self.controlsBox.show()
+			self.separator.hide()
 
-	#=========================================================================	
+	#____________________________________________________________________	
+#=========================================================================	
