@@ -16,6 +16,8 @@ import pwd
 import Globals
 import gettext
 _ = gettext.gettext
+import ProjectTemplate
+import ProjectTemplateDialog
 
 class NewProjectDialog:
 	"""
@@ -39,6 +41,8 @@ class NewProjectDialog:
 		self.signals = {
 			"on_OK_clicked" : self.OnOK,
 			"on_Cancel_clicked" : self.OnCancel,
+			"on_editbutton_clicked" : self.OnEdit,
+			"on_Template_toggled" : self.TemplateCheckboxToggled
 		}
 		
 		self.res.signal_autoconnect(self.signals)
@@ -54,6 +58,24 @@ class NewProjectDialog:
 		self.author = self.res.get_widget("author")
 		self.author.set_activates_default(True)
 		
+		self.template = ProjectTemplate.ProjectTemplate()
+		self.template_checkbox = self.res.get_widget("template_checkbox")
+		self.templatehbox = self.res.get_widget("template_hbox")
+		self.templatehbox.set_sensitive(False)
+		self.templatecombo = self.res.get_widget("template_combo")
+		self.templatecombo.clear()
+		
+		self.templatemodel = gtk.ListStore(str)
+		for files in self.template.LoadDictionaryOfInstrumentsFromTemplateFile().keys():
+			self.templatemodel.append((files,))
+		self.templatecombo.set_model(self.templatemodel)
+		
+		text = gtk.CellRendererText()
+		self.templatecombo.pack_start(text)
+		self.templatecombo.add_attribute(text, "text", 0)
+		
+		self.templatecombo.set_active(0)
+		
 		# Default author to name of currently logged in user
 		try:
 			# Try to get the full name if it exists
@@ -70,12 +92,27 @@ class NewProjectDialog:
 		self.okbutton.grab_default()
 		
 		# Set the default folder of 'folder' (a FileChooserButton)
-		self.folder.set_current_folder(Globals.settings.general["projectfolder"])
+		if os.path.exists(Globals.settings.general["projectfolder"]):
+			self.folder.set_current_folder(Globals.settings.general["projectfolder"])
+		else:
+			self.folder.set_current_folder(os.path.expanduser("~"))
 
 		self.dlg.resize(350, 300)
 		self.dlg.set_icon(self.parent.icon)
 		self.dlg.set_transient_for(self.parent.window)
+
+	#_____________________________________________________________________
 	
+	def TemplateCheckboxToggled(self, widget):
+		"""
+		Called when the template checkbox is activated.
+		The template hbox becomes inactive if the template checkbox is activated.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly. 
+		"""
+		self.templatehbox.set_sensitive(self.template_checkbox.get_active())
+		
 	#_____________________________________________________________________	
 		
 	def OnOK(self, button):
@@ -103,7 +140,7 @@ class NewProjectDialog:
 			folder = "~"
 		
 		try:
-			project = ProjectManager.CreateNewProject(folder,name, author)
+			project = ProjectManager.CreateNewProject(folder, name, author)
 		except ProjectManager.CreateProjectError, e:
 			if e.errno == 1:
 				message = _("Could not initialize project.")
@@ -127,6 +164,7 @@ class NewProjectDialog:
 			dlg.run()
 			dlg.destroy()
 		else:
+			self.AddTemplateToProject(project)
 			self.parent.SetProject(project)
 			self.dlg.destroy()
 		
@@ -141,4 +179,42 @@ class NewProjectDialog:
 		"""
 		self.dlg.destroy()
 
-	#_____________________________________________________________________	
+	#_____________________________________________________________________
+	
+	def OnEdit(self, button):
+		"""
+		Displays the ProjectTemplate dialog, allowing the user to create, delete and modify project
+		templates.
+		
+		Parameters:
+			button -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		ProjectTemplateDialog.ProjectTemplateDialog(self, self.template)
+		
+	#_____________________________________________________________________
+	
+	def ReturnProjectInstrumentTuples(self, name):
+		"""
+		Ths method will return a tuple containing the name, type and pixbuf of the instruments in a given template file.
+		
+		Parameters:
+			name -- the name of the template to use.
+		
+		Returns:
+			instruments -- a list of instruments containing tuples required for project.AddInstruments()
+		"""
+		return self.template.LoadDictionaryOfInstrumentsFromTemplateFile()[name]
+		
+	#_____________________________________________________________________
+	
+	def AddTemplateToProject(self, project):
+		"""
+		Adds template instruments to the user's project.
+		"""
+		if self.template_checkbox.get_active():
+			active = self.templatecombo.get_model()[self.templatecombo.get_active()][0]
+			project.AddInstruments(self.ReturnProjectInstrumentTuples(active))
+	
+	#_____________________________________________________________________
+	
+#=========================================================================
