@@ -14,6 +14,7 @@
 import pygst
 pygst.require("0.10")
 import gst
+import gobject
 import os
 import gzip
 import re
@@ -23,23 +24,59 @@ import UndoSystem
 import Globals
 import xml.dom.minidom as xml
 import Instrument, Event
-from Monitored import Monitored
 import Utils
 import AlsaDevices
+import ProjectManager
 
 #=========================================================================
 
-class Project(Monitored):
+class Project(gobject.GObject):
 	"""
 	This class maintains all of the information required about single Project. It also
 	saves and loads Project files.
 	"""
 	
 	""" The Project structure version. Will be useful for handling old save files. """
-	Globals.VERSION = "0.9"
+	Globals.VERSION = "0.10"
 	
 	""" The audio playback state enum values """
 	AUDIO_STOPPED, AUDIO_RECORDING, AUDIO_PLAYING, AUDIO_PAUSED, AUDIO_EXPORTING = range(5)
+	
+	"""
+	Signals:
+		"audio-state" -- The status of the audio system has changed. See below:
+			"audio-state::play" -- The audio started playing.
+			"audio-state::pause" -- The audio is paused.
+			"audio-state::record" -- The audio started recording.
+			"audio-state::stop" -- The playback or recording was stopped.
+			"audio-state::export-start" -- The audio is being played to a file.
+			"audio-state::export-stop" -- The export to a file has completed.
+		"bpm" -- The beats per minute value was changed.
+		"click-track" -- The click track was turned on or off.
+		"gst-bus-error" -- An error message was posted to the pipeline. Two strings are also send with the error details.
+		"instrument" -- The instruments for this project have changed. The instrument instance will be passed as a parameter. See below:
+			"instrument::added" -- An instrument was added to this project.
+			"instrument::removed" -- An instrument was removed from this project.
+			"instrument::reordered" -- The order of the instruments for this project changed.
+		"time-signature" -- The time signature values were changed.
+		"undo" -- The undo or redo stacks for this project have been changed.
+		"view-start" -- The starting position of the view of this project's timeline has changed.
+		"volume" -- This master volume value for this project has changed.
+		"zoom" -- The zoom level of this project's timeline has changed.
+	"""
+	
+	__gsignals__ = {
+		"audio-state"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, () ),
+		"bpm"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"click-track"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"gst-bus-error"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_STRING) ),
+		"instrument"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,) ),
+		"time-signature"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"undo"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"view-start"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"volume"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"zoom"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () )
+	}
 
 	#_____________________________________________________________________
 
@@ -47,11 +84,13 @@ class Project(Monitored):
 		"""
 		Creates a new instance of Project with default values.
 		"""
-		Monitored.__init__(self)
+		gobject.GObject.__init__(self)
 		
 		self.author = ""			#the author of this project
 		self.name = ""				#the name of this project
 		self.projectfile = ""		#the name of the project file, complete with path
+		self.audio_path = ""
+		self.levels_path = ""
 		self.___id_list = []		#the list of IDs that have already been used, to avoid collisions
 		self.instruments = []		#the list of instruments held by this project
 		self.graveyard = []			# The place where deleted instruments are kept, to later be retrieved by undo functions
@@ -145,8 +184,6 @@ class Project(Monitored):
 		self.transportMode = TransportManager.TransportManager.MODE_BARS_BEATS
 		self.transport = TransportManager.TransportManager(self.transportMode, self)
 
-		self.EOShandlers = [] #extension functions needing EOS notifications
-
 		self.PrepareClick()
 	
 	#_____________________________________________________________________
@@ -211,11 +248,6 @@ class Project(Monitored):
 			self.bus.disconnect(handle)
 
 		self.TerminateRecording()
-
-		#If this is due to end of stream then notify those interested
-		if bus:
-			for function in self.EOShandlers:
-				function()
 		
 		Globals.PrintPipelineDebug("PIPELINE AFTER STOP:", self.mainpipeline)
 		
@@ -271,25 +303,34 @@ class Project(Monitored):
 
 			if channelsNeeded > 1 and not gst.registry_get_default().find_plugin("chansplit"):
 				Globals.debug("Channel splitting element not found when trying to record from multi-input device.")
-				raise AudioInputsError(2)
+				raise ProjectManager.AudioInputsError(2)
 
 			if channelsNeeded > 1: #We're recording from a multi-input device
 				recordingbin = gst.Bin()
 				src = gst.element_factory_make("alsasrc")
 				src.set_property("device", device)
 				
-				capsfilter = gst.element_factory_make("capsfilter")
-				capsString = "audio/x-raw-int,rate=%s" % Globals.settings.recording["samplerate"]
-				caps = gst.caps_from_string(capsString)
-				capsfilter.set_property("caps", caps)
+				capsfilter = None
+				sampleRate = Globals.settings.recording["samplerate"]
+				# 0 means for "autodetect", or more technically "don't use any caps".
+				if sampleRate > 0:
+					gst.element_factory_make("capsfilter")
+					capsString = "audio/x-raw-int,rate=%s" % sampleRate
+					caps = gst.caps_from_string(capsString)
+					capsfilter.set_property("caps", caps)
 				
 				split = gst.element_factory_make("chansplit")
 				
 				recordingbin.add(src)
-				recordingbin.add(capsfilter)
+				if capsfilter:
+					recordingbin.add(capsfilter)
 				recordingbin.add(split)
-				src.link(capsfilter)
-				capsfilter.link(split)
+				
+				if capsfilter:
+					src.link(capsfilter)
+					capsfilter.link(split)
+				else:
+					src.link(split)
 				
 				split.connect("pad-added", self.__RecordingPadAddedCb, recInstruments, recordingbin)
 				Globals.debug("Recording in multi-input mode")
@@ -300,8 +341,19 @@ class Project(Monitored):
 				event = instr.GetRecordingEvent()
 				
 				encodeString = Globals.settings.recording["fileformat"]
-				capsString = "audio/x-raw-int,rate=%s" % Globals.settings.recording["samplerate"]
-				pipe = "alsasrc device=%s ! %s ! audioconvert ! level name=recordlevel interval=%d" +\
+				
+				sampleRate = 0
+				try:
+					sampleRate = int( Globals.settings.recording["samplerate"] )
+				except ValueError:
+					pass
+				# 0 means for "autodetect", or more technically "don't use any caps".
+				if sampleRate > 0:
+					capsString = "audio/x-raw-int,rate=%s ! audioconvert" % sampleRate
+				else:
+					capsString = "audioconvert"
+					
+				pipe = "alsasrc device=%s ! %s ! level name=recordlevel interval=%d" +\
 							" ! audioconvert ! %s ! filesink location=%s"
 				pipe %= (device, capsString, event.LEVEL_INTERVAL * gst.SECOND, encodeString, event.file.replace(" ", "\ "))
 				
@@ -335,6 +387,16 @@ class Project(Monitored):
 					for mp3: "lame"
 					for wav: "wavenc"
 		"""
+		#try to create encoder/muxer first, before modifying the main pipeline.
+		try:
+			self.encodebin = gst.parse_bin_from_description("audioconvert ! %s" % encodeBin, True)
+		except gobject.GError, e:
+			if e.code == gst.PARSE_ERROR_NO_SUCH_ELEMENT:
+				error_no = ProjectManager.ProjectExportException.MISSING_ELEMENT
+			else:
+				error_no = ProjectManager.ProjectExportException.INVALID_ENCODE_BIN
+			raise ProjectManager.ProjectExportException(error_no, e.message)
+
 		#stop playback because some elements will be removed from the pipeline
 		self.Stop()
 		
@@ -347,9 +409,7 @@ class Project(Monitored):
 		self.outfile = gst.element_factory_make("filesink", "export_file")
 		self.outfile.set_property("location", filename)
 		self.playbackbin.add(self.outfile)
-		
-		#create encoder/muxer
-		self.encodebin = gst.gst_parse_bin_from_description("audioconvert ! %s" % encodeBin, True)
+
 		self.playbackbin.add(self.encodebin)
 		self.levelElementCaps.link(self.encodebin)
 		self.encodebin.link(self.outfile)
@@ -363,7 +423,7 @@ class Project(Monitored):
 		self.exportFilename = filename
 		#start the pipeline!
 		self.Play(newAudioState=self.AUDIO_EXPORTING)
-		self.StateChanged("export-start")
+		self.emit("audio-state::export-start")
 
 	#_____________________________________________________________________
 	
@@ -401,7 +461,7 @@ class Project(Monitored):
 		self.levelElementCaps.link(self.levelElement)
 		self.levelElement.link(self.masterSink)
 		
-		self.StateChanged("export-stop")
+		self.emit("audio-state::export-stop")
 	
 	#_____________________________________________________________________
 	
@@ -462,13 +522,13 @@ class Project(Monitored):
 		"""
 		self.audioState = newState
 		if newState == self.AUDIO_PAUSED:
-			self.StateChanged("pause")
+			self.emit("audio-state::pause")
 		elif newState == self.AUDIO_PLAYING:
-			self.StateChanged("play")
+			self.emit("audio-state::play")
 		elif newState == self.AUDIO_STOPPED:
-			self.StateChanged("stop")
+			self.emit("audio-state::stop")
 		elif newState == self.AUDIO_RECORDING:
-			self.StateChanged("record")
+			self.emit("audio-state::record")
 		elif newState == self.AUDIO_EXPORTING:
 			self.exportPending = False
 		
@@ -499,7 +559,7 @@ class Project(Monitored):
 							"audioconvert ! %s ! filesink location=%s"
 				pipe %= (event.LEVEL_INTERVAL, encodeString, event.file.replace(" ", "\ "))
 				
-				encodeBin = gst.gst_parse_bin_from_description(pipe, True)
+				encodeBin = gst.parse_bin_from_description(pipe, True)
 				bin.add(encodeBin)
 				pad.link(encodeBin.get_pad("sink"))
 				
@@ -572,12 +632,17 @@ class Project(Monitored):
 		"""
 		error, debug = message.parse_error()
 		
+		#FIXME: remove these prints!
+		print "*Project Error*"
+		print ("Code: %s, Domain: %s")%(error.code, error.domain)
+		print "Message: "+error.message
+		
 		Globals.debug("Gstreamer bus error:", str(error), str(debug))
-		self.StateChanged("gst-bus-error", str(error), str(debug))
+		self.emit("gst-bus-error", str(error), str(debug))
 
 	#_____________________________________________________________________
 	
-	def SaveProjectFile(self, path=None):
+	def SaveProjectFile(self, path=None, backup=False):
 		"""
 		Saves the Project and its children as an XML file
 		to the path specified by file.
@@ -590,20 +655,38 @@ class Project(Monitored):
 			if not self.projectfile:
 				raise "No save path specified!"
 			path = self.projectfile
+		
+		if not self.audio_path:
+			self.audio_path = os.path.join(os.path.dirname(path), "audio")
+		if not self.levels_path:
+			self.levels_path = os.path.join(os.path.dirname(path), "levels")
 			
+		if os.path.exists(self.audio_path):
+			if not os.path.isdir(self.audio_path):
+				raise "Audio save location is not a directory"
+		else:
+			os.mkdir(self.audio_path)
+		
+		if os.path.exists(self.levels_path):
+			if not os.path.isdir(self.levels_path):
+				raise "Levels save location is not a directory"
+		else:
+			os.mkdir(self.levels_path)
+		
 		if not path.endswith(".jokosher"):
 			path = path + ".jokosher"
 			
 		#sync the transport's mode with the one which will be saved
 		self.transportMode = self.transport.mode
 		
-		self.unsavedChanges = False
-		#purge main undo stack so that it will not prompt to save on exit
-		self.__savedUndoStack.extend(self.__undoStack)
-		self.__undoStack = []
-		#purge savedRedoStack so that it will not prompt to save on exit
-		self.__redoStack.extend(self.__savedRedoStack)
-		self.__savedRedoStack = []
+		if not backup:
+			self.unsavedChanges = False
+			#purge main undo stack so that it will not prompt to save on exit
+			self.__savedUndoStack.extend(self.__undoStack)
+			self.__undoStack = []
+			#purge savedRedoStack so that it will not prompt to save on exit
+			self.__redoStack.extend(self.__savedRedoStack)
+			self.__savedRedoStack = []
 		
 		doc = xml.Document()
 		head = doc.createElement("JokosherProject")
@@ -614,7 +697,8 @@ class Project(Monitored):
 		params = doc.createElement("Parameters")
 		head.appendChild(params)
 		
-		items = ["viewScale", "viewStart", "name", "author", "transportMode", "bpm", "meter_nom", "meter_denom"]
+		items = ["viewScale", "viewStart", "name", "author",
+		         "transportMode", "bpm", "meter_nom", "meter_denom", "projectfile"]
 		
 		Utils.StoreParametersToXML(self, doc, params, items)
 			
@@ -649,7 +733,7 @@ class Project(Monitored):
 			#if the saving doesn't fail, move it to the proper location
 			os.rename(path + "~", path)		
 		
-		self.StateChanged("undo")
+		self.emit("undo")
 	
 	#_____________________________________________________________________
 
@@ -663,8 +747,6 @@ class Project(Monitored):
 				os.remove(file)
 		self.deleteOnCloseAudioFiles = []
 		
-		self.ClearListeners()
-		self.transport.ClearListeners()
 		self.mainpipeline.set_state(gst.STATE_NULL)
 		
 	#_____________________________________________________________________
@@ -735,7 +817,7 @@ class Project(Monitored):
 				#since there is no other record that something has 
 				#changed after savedRedoStack is purged
 				self.unsavedChanges = True
-		self.StateChanged("undo")
+		self.emit("undo")
 	
 	#_____________________________________________________________________
 	
@@ -836,7 +918,9 @@ class Project(Monitored):
 		self.temp = self.bpm
 		if self.bpm != bpm:
 			self.bpm = bpm
-			self.StateChanged("bpm")
+			#FIXME: find a better way to do project.PrepareClick() it doesn't take a really long time with large bpm
+			self.PrepareClick()
+			self.emit("bpm")
 	
 	#_____________________________________________________________________
 
@@ -862,7 +946,7 @@ class Project(Monitored):
 		if self.meter_nom != nom or self.meter_denom != denom:
 			self.meter_nom = nom
 			self.meter_denom = denom
-			self.StateChanged("time-signature")
+			self.emit("time-signature")
 			
 	#_____________________________________________________________________
 	
@@ -880,7 +964,7 @@ class Project(Monitored):
 		"""
 		
 		undoAction = self.NewAtomicUndoAction()
-		for name, type, pixbuf, path in instrTuples:
+		for name, type, pixbuf in instrTuples:
 			self.AddInstrument(name, type, pixbuf, _undoAction_=undoAction)
 	
 	#_____________________________________________________________________
@@ -923,12 +1007,11 @@ class Project(Monitored):
 		if len(self.instruments) == 0:
 			#If this is the first instrument, arm it by default
 			instr.isArmed = True
-		audio_dir = os.path.join(os.path.split(self.projectfile)[0], "audio")
-		instr.path = os.path.join(audio_dir)
 		
 		self.temp = instr.id
 		self.instruments.append(instr)
 		
+		self.emit("instrument::added", instr)
 		return instr
 		
 	#_____________________________________________________________________	
@@ -963,6 +1046,7 @@ class Project(Monitored):
 			event.StopGenerateWaveform(False)
 			
 		self.temp = id
+		self.emit("instrument::removed", instr)
 	
 	#_____________________________________________________________________
 	
@@ -990,6 +1074,7 @@ class Project(Monitored):
 		instr.isVisible = True
 		self.graveyard.remove(instr)
 		self.temp = id
+		self.emit("instrument::added", instr)
 		
 	#_____________________________________________________________________
 	
@@ -1009,7 +1094,8 @@ class Project(Monitored):
 		self.temp1 = self.instruments.index(instr)
 		
 		self.instruments.remove(instr)
-		self.instruments.insert(position, instr)		
+		self.instruments.insert(position, instr)
+		self.emit("instrument::reordered", instr)
 	
 	#_____________________________________________________________________
 	
@@ -1067,7 +1153,7 @@ class Project(Monitored):
 		start = max(0, min(self.GetProjectLength(), start))
 		if self.viewStart != start:
 			self.viewStart = start
-			self.StateChanged("view-start")
+			self.emit("view-start")
 		
 	#_____________________________________________________________________
 	
@@ -1079,7 +1165,7 @@ class Project(Monitored):
 			scale -- view scale in pixels per second.
 		"""
 		self.viewScale = scale
-		self.StateChanged("zoom")
+		self.emit("zoom")
 		
 	#_____________________________________________________________________
 
@@ -1108,12 +1194,13 @@ class Project(Monitored):
 			
 	#_____________________________________________________________________
 	
-	def GenerateUniqueID(self, id = None):
+	def GenerateUniqueID(self, id = None,  reserve=True):
 		"""
 		Creates a new unique ID which can be assigned to an new Project object.
 		
 		Parameters:
 			id -- an unique ID proposal. If it's already taken, a new one is generated.
+			reserve -- if True, the ID will be recorded and never returned again.
 			
 		Returns:
 			an unique ID suitable for a new Project.
@@ -1144,7 +1231,7 @@ class Project(Monitored):
 		self.volume = volume
 		for instr in self.instruments:
 			instr.UpdateVolume()
-		self.StateChanged("volume")
+		self.emit("volume")
 
 	#_____________________________________________________________________
 
@@ -1200,23 +1287,17 @@ class Project(Monitored):
 
 	#_____________________________________________________________________
 
-	def EnableClick(self):
+	def SetClickEnabled(self, active):
 		"""
 		Unmutes and enables the click track.
+		
+		Parameters:
+			active -- If True, the click track will be activated, otherwise it will be disabled.
 		"""
-	
-		self.clickTrackVolume.set_property("mute", False)
-		self.clickEnabled = True
-
-	#_____________________________________________________________________
-
-	def DisableClick(self):
-		"""
-		Mutes and disables the click track.
-		"""
-	
-		self.clickTrackVolume.set_property("mute", True)
-		self.clickEnabled = False
+		if self.clickEnabled != active:
+			self.clickTrackVolume.set_property("mute", not active)
+			self.clickEnabled = active
+			self.emit("click-track")
 
 	#_____________________________________________________________________
 
@@ -1285,7 +1366,7 @@ class Project(Monitored):
 		
 		elif sinkString != "autoaudiosink":
 			try:
-				sinkElement = gst.gst_parse_bin_from_description(sinkString, True)
+				sinkElement = gst.parse_bin_from_description(sinkString, True)
 			except gobject.GError:
 				Globals.debug("Parsing failed: %s" % sinkString)
 			else:
@@ -1297,32 +1378,9 @@ class Project(Monitored):
 			Globals.debug("Using autoaudiosink for audio output")
 			
 		return sinkElement
-	
-	#_____________________________________________________________________
-	
-	def AddEndOfStreamHandler(self, function):
-		"""
-		Adds a function to the list of functions that need notification of 
-		end-of-stream messages from gstreamer
-		
-		Parameters:
-			function -- the function to be added
-		"""
-		self.EOShandlers.append(function)
-		
-	#_____________________________________________________________________
-	
-	def RemoveEndOfStreamHandler(self, function):
-		"""
-		Removes a function from the list of functions that need notification of 
-		end-of-stream messages from gstreamer
-		
-		Parameters:
-			function -- the function to be removed
-		"""
-		self.EOShandlers.remove(function)
 		
 	#____________________________________________________________________	
+	
 	def GetInputFilenames(self):
 		"""
 		Obtains a list of  all filenames that are to be input to
