@@ -10,7 +10,7 @@
 #=========================================================================
 
 import urlparse, os, gzip, shutil, gst
-import Globals, Utils, UndoSystem
+import Globals, Utils, UndoSystem, LevelsList
 import Project, Instrument, Event
 import xml.dom.minidom as xml
 import traceback
@@ -52,14 +52,16 @@ def CreateNewProject(projecturi, name, author):
 	project.name = name
 	project.author = author
 	project.projectfile = os.path.join(projectdir, filename)
+	project.audio_path = os.path.join(projectdir, "audio")
+	project.levels_path = os.path.join(projectdir, "levels")
 
 	if os.path.exists(projectdir):
 		raise CreateProjectError(2)
 	else: 
-		audio_dir = os.path.join(projectdir, "audio")
 		try:
 			os.mkdir(projectdir)
-			os.mkdir(audio_dir)
+			os.mkdir(project.audio_path)
+			os.mkdir(project.levels_path)
 		except:
 			raise CreateProjectError(3)
 
@@ -118,8 +120,16 @@ def LoadProjectFile(uri):
 		raise OpenProjectError(4, projectfile)
 
 	try:
-		gzipfile = gzip.GzipFile(projectfile, "r")
-		doc = xml.parse(gzipfile)
+		try:
+			gzipfile = gzip.GzipFile(projectfile, "r")
+			doc = xml.parse(gzipfile)
+		except IOError, e:
+			if e.message == "Not a gzipped file":
+				# starting from 0.10, we accept both gzipped xml and plain xml
+				file_ = open(projectfile, "r")
+				doc = xml.parse(file_)
+			else:
+				raise e
 	except Exception, e:
 		Globals.debug(e.__class__, e)
 		# raise "This file doesn't unzip" message
@@ -127,6 +137,9 @@ def LoadProjectFile(uri):
 	
 	project = Project.Project()
 	project.projectfile = projectfile
+	projectdir = os.path.split(projectfile)[0]
+	project.audio_path = os.path.join(projectdir, "audio")
+	project.levels_path = os.path.join(projectdir, "levels")
 	
 	#only open projects with the proper version number
 	version = None
@@ -147,6 +160,7 @@ def LoadProjectFile(uri):
 			#if we're loading an old version copy the project so that it is not overwritten when the user clicks save
 			withoutExt = os.path.splitext(projectfile)[0]
 			shutil.copy(projectfile, "%s.%s.jokosher" % (withoutExt, version))
+		project.projectfile = projectfile
 		return project
 	else:
 		# raise a "this project was created in an incompatible version of Jokosher" message
@@ -202,6 +216,7 @@ class _LoadZPOFile:
 				id = None
 			e = Event.Event(instr, None, id)
 			self.LoadEvent(e, ev)
+			e.levels_file = os.path.basename(e.file + Event.Event.LEVELS_FILE_EXTENSION)
 			instr.events.append(e)
 		
 		pixbufFilename = os.path.basename(instr.pixbufPath)
@@ -242,19 +257,7 @@ class _LoadZPOFile:
 					value = float(n.getAttribute("fade"))
 					event._Event__fadePointsDict[pos] = value
 		
-		try:	
-			levelsXML = xmlNode.getElementsByTagName("Levels")[0]
-		except IndexError:
-			Globals.debug("No event levels in project file")
-			event.GenerateWaveform()
-		else: 
-			if levelsXML.nodeType == xml.Node.ELEMENT_NODE:
-				value = str(levelsXML.getAttribute("value"))
-				event.levels = map(float, value.split(","))
-		
-		if event.isLoading:
-			event.GenerateWaveform()
-
+		event.GenerateWaveform()
 		event._Event__UpdateAudioFadePoints()
 		event.CreateFilesource()
 	
@@ -296,6 +299,8 @@ class _LoadZPTFile:
 						functionString = str(cmdNode.getAttribute("function"))  
 						paramList = Utils.LoadListFromXML(cmdNode)  
 					
+						functionString = ApplyUndoCompat(objectString, functionString, "0.2")
+						
 						undoAction = UndoSystem.AtomicUndoAction()  
 						undoAction.AddUndoCommand(objectString, functionString, paramList)  
 						stack.append(undoAction)  
@@ -335,9 +340,6 @@ class _LoadZPTFile:
 		
 		Utils.LoadParametersFromXML(instr, params)
 		
-		#figure out the instrument's path based on the location of the projectfile
-		instr.path = os.path.join(os.path.dirname(self.project.projectfile), "audio")
-		
 		globaleffect = xmlNode.getElementsByTagName("GlobalEffect")
 		
 		for effect in globaleffect:
@@ -356,18 +358,18 @@ class _LoadZPTFile:
 				id = None
 			event = Event.Event(instr, None, id)
 			self.LoadEvent(event, ev)
+			event.levels_file = os.path.basename(event.file + Event.Event.LEVELS_FILE_EXTENSION)
 			instr.events.append(event)
 		
-		for ev in xmlNode.getElementsByTagName("DeadEvent"):  
-			try:  
-				id = int(ev.getAttribute("id"))  
-			except ValueError:  
-				id = None  
-			event = Event.Event(instr, None, id)  
-			self.LoadEvent(event, ev)
-			instr.graveyard.append(event)  
-			#remove it from the composition so it doesnt play  
-			instr.composition.remove(event.filesrc) 
+		for ev in xmlNode.getElementsByTagName("DeadEvent"):
+			try:
+				id = int(ev.getAttribute("id"))
+			except ValueError:
+				id = None
+			event = Event.Event(instr, None, id)
+			self.LoadEvent(event, ev, True)
+			event.levels_file = os.path.basename(event.file + Event.Event.LEVELS_FILE_EXTENSION)
+			instr.graveyard.append(event)
 
 		#load image from file based on unique type
 		for instrTuple in Globals.getCachedInstruments():
@@ -381,10 +383,12 @@ class _LoadZPTFile:
 		instr.panElement.set_property("panorama", instr.pan)
 		#check if instrument is muted and setup accordingly
 		instr.OnMute()
+		#update the volume element with the newly loaded value
+		instr.UpdateVolume()
 		
 	#_____________________________________________________________________
 		
-	def LoadEvent(self, event, xmlNode):
+	def LoadEvent(self, event, xmlNode, isDead=False):
 		"""
 		Restores an Event from its version 0.2 XML representation.
 		
@@ -398,7 +402,7 @@ class _LoadZPTFile:
 		
 		if not os.path.isabs(event.file):
 			# If there is a relative path for event.file, assume it is in the audio dir
-			event.file = os.path.join(event.instrument.path, event.file)
+			event.file = os.path.join(event.instrument.project.audio_path, event.file)
 		
 		try:
 			xmlPoints = xmlNode.getElementsByTagName("FadePoints")[0]
@@ -406,27 +410,20 @@ class _LoadZPTFile:
 			Globals.debug("Missing FadePoints in Event XML")
 		else:
 			event._Event__fadePointsDict = Utils.LoadDictionaryFromXML(xmlPoints)
-		
-		try:	
-			levelsXML = xmlNode.getElementsByTagName("Levels")[0]
-		except IndexError:
-			Globals.debug("No event levels in project file")
-			event.GenerateWaveform()
-		else: 
-			if levelsXML.nodeType == xml.Node.ELEMENT_NODE:
-				value = str(levelsXML.getAttribute("value"))
-				event.levels = map(float, value.split(","))
 
-		if event.isLoading or event.isRecording:
+		if not isDead:
+			#if event.isLoading or event.isRecording:
+			# we have to always generate waveform because 0.10 uses different levels format
 			event.GenerateWaveform()
-
-		event._Event__UpdateAudioFadePoints()
-		event.CreateFilesource()
+			event._Event__UpdateAudioFadePoints()
+			event.CreateFilesource()
 	
 	#_____________________________________________________________________
 #=========================================================================
 
-class _LoadZPNFile:
+class _LoadZPNFile(_LoadZPTFile):
+	LOADING_VERSION = "0.9"
+	
 	def __init__(self, project, xmlDoc):
 		"""
 		Loads a Jokosher version 0.9 (Zero Point Nine) Project file into
@@ -482,111 +479,7 @@ class _LoadZPNFile:
 			instr.RemoveAndUnlinkPlaybackbin()
 	
 	#_____________________________________________________________________
-	
-	def LoadInstrument(self, instr, xmlNode):
-		"""
-		Restores an Instrument from version 0.9 XML representation.
-		
-		Parameters:
-			instr -- the Instrument instance to apply loaded properties to.
-			xmlNode -- the XML node to retreive data from.
-		"""
-		params = xmlNode.getElementsByTagName("Parameters")[0]
-		
-		Utils.LoadParametersFromXML(instr, params)
-		
-		#figure out the instrument's path based on the location of the projectfile
-		instr.path = os.path.join(os.path.dirname(self.project.projectfile), "audio")
-		
-		globaleffect = xmlNode.getElementsByTagName("GlobalEffect")
-		
-		for effect in globaleffect:
-			elementname = str(effect.getAttribute("element"))
-			Globals.debug("Loading effect:", elementname)
-			gstElement = instr.AddEffect(elementname)
-			
-			propsdict = Utils.LoadDictionaryFromXML(effect)
-			for key, value in propsdict.iteritems():
-				gstElement.set_property(key, value)		
-			
-		for ev in xmlNode.getElementsByTagName("Event"):
-			try:
-				id = int(ev.getAttribute("id"))
-			except ValueError:
-				id = None
-			event = Event.Event(instr, None, id)
-			self.LoadEvent(event, ev)
-			instr.events.append(event)
-	
-		for ev in xmlNode.getElementsByTagName("DeadEvent"):
-			try:
-				id = int(ev.getAttribute("id"))
-			except ValueError:
-				id = None
-			event = Event.Event(instr, None, id)
-			self.LoadEvent(event, ev)
-			instr.graveyard.append(event)
-			#remove it from the composition so it doesnt play
-			instr.composition.remove(event.filesrc)
-		
-		#load image from file based on unique type
-		for instrTuple in Globals.getCachedInstruments():
-			if instr.instrType == instrTuple[1]:
-				instr.pixbuf = instrTuple[2]
-				break
-		if not instr.pixbuf:
-			Globals.debug("Error, could not load image:", instr.instrType)
-		
-		# load pan level
-		instr.panElement.set_property("panorama", instr.pan)
-		#check if instrument is muted and setup accordingly
-		instr.OnMute()
-		#update the volume element with the newly loaded value
-		instr.UpdateVolume()
-		
-	#_____________________________________________________________________
-		
-	def LoadEvent(self, event, xmlNode):
-		"""
-		Restores an Event from its version 0.9 XML representation.
-		
-		Parameters:
-			event -- the Event instance to apply loaded properties to.
-			xmlNode -- the XML node to retreive data from.
-		"""
-		params = xmlNode.getElementsByTagName("Parameters")[0]
-		
-		Utils.LoadParametersFromXML(event, params)
-		
-		if not os.path.isabs(event.file):
-			# If there is a relative path for event.file, assume it is in the audio dir
-			event.file = os.path.join(event.instrument.path, event.file)
-		
-		try:
-			xmlPoints = xmlNode.getElementsByTagName("FadePoints")[0]
-		except IndexError:
-			Globals.debug("Missing FadePoints in Event XML")
-		else:
-			event._Event__fadePointsDict = Utils.LoadDictionaryFromXML(xmlPoints)
-		
-		try:	
-			levelsXML = xmlNode.getElementsByTagName("Levels")[0]
-		except IndexError:
-			Globals.debug("No event levels in project file")
-			event.GenerateWaveform()
-		else: 
-			if levelsXML.nodeType == xml.Node.ELEMENT_NODE:
-				value = str(levelsXML.getAttribute("value"))
-				event.levels = map(float, value.split(","))
 
-		if event.isLoading or event.isRecording:
-			event.GenerateWaveform()
-
-		event._Event__UpdateAudioFadePoints()
-		event.CreateFilesource()
-	
-	#_____________________________________________________________________
-	
 	def LoadUndoAction(self, undoAction, xmlNode):
 		"""
 		Loads an AtomicUndoAction from an XML node.
@@ -605,11 +498,57 @@ class _LoadZPNFile:
 				functionString = str(cmdNode.getAttribute("function"))
 				paramList = Utils.LoadListFromXML(cmdNode)
 				
+				functionString = ApplyUndoCompat(objectString, functionString, self.LOADING_VERSION)
+				
 				undoAction.AddUndoCommand(objectString, functionString, paramList)
 		
 	#_____________________________________________________________________
 #=========================================================================
 
+class _LoadZPTenFile(_LoadZPNFile):
+	LOADING_VERSION = "0.10"
+	
+	def LoadEvent(self, event, xmlNode, isDead=False):
+		"""
+		Restores an Event from its version 0.10 XML representation.
+		
+		Parameters:
+			event -- the Event instance to apply loaded properties to.
+			xmlNode -- the XML node to retreive data from.
+		"""
+		params = xmlNode.getElementsByTagName("Parameters")[0]
+		
+		Utils.LoadParametersFromXML(event, params)
+		
+		if not os.path.isabs(event.file):
+			# If there is a relative path for event.file, assume it is in the audio dir
+			event.file = os.path.join(self.project.audio_path, event.file)
+		
+		try:
+			xmlPoints = xmlNode.getElementsByTagName("FadePoints")[0]
+		except IndexError:
+			Globals.debug("Missing FadePoints in Event XML")
+		else:
+			event._Event__fadePointsDict = Utils.LoadDictionaryFromXML(xmlPoints)
+
+		if not isDead:
+			if event.isLoading or event.isRecording:  
+				event.GenerateWaveform()
+			else:
+				levels_path = os.path.join(self.project.levels_path, event.levels_file)
+				try:
+					event.levels_list.fromfile(levels_path)
+				except LevelsList.CorruptFileError:
+					Globals.debug("Cannot load levels from file", levels_path)
+				if not event.levels_list:
+					event.GenerateWaveform()
+			event._Event__UpdateAudioFadePoints()
+			event.CreateFilesource()
+	
+	#_____________________________________________________________________
+
+#=========================================================================
+
 class OpenProjectError(EnvironmentError):
 	"""
 	This class will get created when a opening a Project fails.
@@ -720,6 +659,65 @@ class InvalidProjectError(Exception):
 
 #=========================================================================
 
-JOKOSHER_VERSION_FUNCTIONS = {"0.1" : _LoadZPOFile, "0.2" : _LoadZPTFile, "0.9" : _LoadZPNFile}
+class ProjectExportException(Exception):
+	"""
+	This class will get created when there are problems with the soundcard inputs.
+	It's used for handling errors.
+
+	Error Codes:
+	MISSING_ELEMENT - incorrect element name or element not installed.
+	INVALID_ENCODE_BIN - invalid bin description; invalid syntax, invalid properties, incompatible caps, etc.
+	"""
+	MISSING_ELEMENT, INVALID_ENCODE_BIN = range(2)
+	
+	#_____________________________________________________________________
+	
+	def __init__(self, errno, message):
+		"""
+		Creates a new instance of AudioInputsError.
+		
+		Parameters:
+			errno -- number indicating the type of error. See error codes above.
+		"""
+		Exception.__init__(self)
+		self.errno = errno
+		self.message = message
+		
+	#_____________________________________________________________________
+
+#=========================================================================
+
+def ApplyUndoCompat(objectString, functionString, version):
+	if UNDO_COMPAT_DICT.has_key(version):
+		compact_dict = UNDO_COMPAT_DICT[version]
+		tuple_ = (objectString[0], functionString)
+		if compact_dict.has_key(tuple_):
+			return compact_dict[tuple_]
+	
+	return functionString
+
+#=========================================================================
+
+JOKOSHER_VERSION_FUNCTIONS = {
+	"0.1" : _LoadZPOFile, 
+	"0.2" : _LoadZPTFile, 
+	"0.9" : _LoadZPNFile,
+	"1.0" : _LoadZPNFile,  # 1.0 was never used in a release, and it identical to 0.9
+	"0.10" : _LoadZPTenFile,
+}
+
+zero_nine_compat = {("E", "Move") : "_Compat09_Move"}
+zero_two_compat = {
+	("E", "Split") : "_Compat02_Split",
+	("E", "Join") : "_Compat02_Join",
+	("E", "UndoTrim") : "_Compat02_UndoTrim",
+}
+#we can't import undo from 0.1 because the storage of undo was revamped for version 0.2
+
+#all the compat info from newer versions, also applies to older versions
+for key, value in zero_nine_compat.iteritems():
+	zero_two_compat.setdefault(key, value)
+
+UNDO_COMPAT_DICT = {"0.2" : zero_two_compat, "0.9" : zero_nine_compat}
 
 #=========================================================================
