@@ -22,9 +22,6 @@ class RecordingView(gtk.Frame):
 	"""
 	This class encapsulates a visual layout of a project comprising
 	instrument tracks, timeline, and horizontal scrollbars.
-	Despite its name, it also appears under the mixing view contained
-	in a CompactMixView object, where it represents the same
-	information with shorter instrument tracks.
 	"""
 	
 	""" GTK widget name """
@@ -51,25 +48,19 @@ class RecordingView(gtk.Frame):
 
 	#_____________________________________________________________________
 
-	def __init__(self, project, mainview, mixView=None, small=False):
+	def __init__(self, project, mainview, small=False):
 		"""
 		Creates a new instance of RecordingView.
 		
 		Parameters:
 			project -- the currently active Project.
 			mainview -- the main Jokosher window (MainApp).
-			mixView -- the CompactMixView object that holds this instance of
-						RecordingView, if the mixing view is the currently 
-						active one.
-						If the recording view is the active one, then this
-						should be set to None.
 			small -- set to True if we want small edit views (i.e. for the mixing view).
 		"""
 		gtk.Frame.__init__(self)
 
 		self.project = project
 		self.mainview = mainview
-		self.mixView = mixView
 		self.small = small
 		self.timelinebar = TimeLineBar.TimeLineBar(self.project, self, mainview)
 		
@@ -103,35 +94,36 @@ class RecordingView(gtk.Frame):
 		
 		self.lastzoom = 0
 		
-		#recording view contains zoom buttons
-		if not self.mixView:
-			self.zoomSlider = gtk.HScale()
-			self.zoomSlider.set_size_request(70, -1)
-			
-			self.zoomSlider.set_range(self.ZOOM_MIN_SCALE, self.ZOOM_MAX_SCALE)
-			self.zoomSlider.set_increments(0.2, 0.2)
-			self.zoomSlider.set_draw_value(False)
-			self.zoomSlider.set_value(self.project.viewScale)
-			self.zoomtip = gtk.Tooltips()
-			self.zoomtip.set_tip(self.zoomSlider, _("Zoom the timeline"),None)
-			
-			self.zoomSlider.connect("value-changed", self.OnZoom)
-			
-			inbutton = gtk.Button()
-			inimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
-			inbutton.set_image(inimg)
-			inbutton.set_relief(gtk.RELIEF_NONE)
-			inbutton.connect("clicked", self.OnZoomIn)
-			
-			outbutton = gtk.Button()
-			outimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
-			outbutton.set_image(outimg)
-			outbutton.set_relief(gtk.RELIEF_NONE)
-			outbutton.connect("clicked", self.OnZoomOut)
+		self.zoomSlider = gtk.HScale()
+		self.zoomSlider.set_size_request(70, -1)
+		
+		self.zoomSlider.set_range(self.ZOOM_MIN_SCALE, self.ZOOM_MAX_SCALE)
+		self.zoomSlider.set_increments(0.2, 0.2)
+		self.zoomSlider.set_draw_value(False)
+		self.zoomSlider.set_value(self.project.viewScale)
+		self.zoomtip = gtk.Tooltips()
+		self.zoomtip.set_tip(self.zoomSlider, _("Zoom the timeline - Right-Click to reset to the default level"), None)
+		
+		self.zoomSlider.connect("value-changed", self.OnZoom)
+		self.zoomSlider.connect("button-press-event", self.OnZoomReset)
+		
+		self.inbutton = gtk.Button()
+		inimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
+		self.inbutton.set_image(inimg)
+		self.inbutton.set_relief(gtk.RELIEF_NONE)
+		self.zoomtip.set_tip(self.inbutton, _("Zoom in timeline"), None)
+		self.inbutton.connect("clicked", self.OnZoomIn)
+		
+		self.outbutton = gtk.Button()
+		outimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
+		self.outbutton.set_image(outimg)
+		self.outbutton.set_relief(gtk.RELIEF_NONE)
+		self.zoomtip.set_tip(self.outbutton, _("Zoom out timeline"), None)
+		self.outbutton.connect("clicked", self.OnZoomOut)
 
-			self.hb.pack_start( outbutton, False, False)
-			self.hb.pack_start( self.zoomSlider, False, False)
-			self.hb.pack_start( inbutton, False, False)
+		self.hb.pack_start( self.outbutton, False, False)
+		self.hb.pack_start( self.zoomSlider, False, False)
+		self.hb.pack_start( self.inbutton, False, False)
 		
 		self.extraScrollTime = 25
 		self.centreViewOnPosition = False
@@ -146,13 +138,28 @@ class RecordingView(gtk.Frame):
 		self.connect("button_press_event", self.OnMouseDown)
 		self.connect("size-allocate", self.OnAllocate)
 		
+		#connect to the project signals
+		self.project.connect("instrument::added", self.OnInstrumentAdded)
+		self.project.connect("instrument::reordered", self.OnInstrumentReordered)
+		self.project.connect("instrument::removed", self.OnInstrumentRemoved)
+		
 		self.vbox.drag_dest_set(	gtk.DEST_DEFAULT_DROP,
 									self.DRAG_TARGETS, 
 									gtk.gdk.ACTION_COPY)
 		self.vbox.connect("drag_data_received", self.OnDragDataReceived)
 		self.vbox.connect("drag_motion", self.OnDragMotion)
 		
-		self.Update()
+		#add the instruments that were loaded from the project file already
+		for instr in self.project.instruments:
+			self.OnInstrumentAdded(project, instr)
+			
+		self.show_all()
+		self.show_all()
+		if self.small:
+			self.inbutton.hide()
+			self.outbutton.hide()
+			self.zoomSlider.hide()
+		
 	#_____________________________________________________________________
 
 	def OnExpose(self, widget, event):
@@ -200,13 +207,12 @@ class RecordingView(gtk.Frame):
 		elif self.project.viewStart + self.scrollRange.page_size > length:
 			self.SetViewPosition(length - self.scrollRange.page_size)
 		
-		if not self.mixView:
-			#check the min zoom value (based on project length)
-			pixelSize = self.allocation.width - Globals.INSTRUMENT_HEADER_WIDTH - 4	# four pixels to account for borders
-			minScale = pixelSize / length
-			self.zoomSlider.set_range(minScale, self.ZOOM_MAX_SCALE)
-			if self.zoomSlider.get_value() < minScale:
-				self.zoomSlider.set_value(minScale)
+		#check the min zoom value (based on project length)
+		pixelSize = self.allocation.width - Globals.INSTRUMENT_HEADER_WIDTH - 4	# four pixels to account for borders
+		minScale = pixelSize / length
+		self.zoomSlider.set_range(minScale, self.ZOOM_MAX_SCALE)
+		if self.zoomSlider.get_value() < minScale:
+			self.zoomSlider.set_value(minScale)
 		
 	#_____________________________________________________________________
 
@@ -222,84 +228,82 @@ class RecordingView(gtk.Frame):
 		
 	#_____________________________________________________________________
 	
-
-	def Update(self):
+	def OnInstrumentAdded(self, project, instrument):
 		"""
-		Updates the GUI to reflect changes on the instruments, timeline and
-		scrollbars.
-		Called either directly from OnStateChanged(), or via the owning
-		CompactMixView.update()(depending on which view we are in) when
-		there is a change of state in an instrument being listened to.
+		Callback for when an instrument is added to the project.
 		
-		Considerations:
-			InstrumentViews MUST have the order that the instruments have in
-			Project.instruments, to keep the drag and drop of InstrumentViews
-			consistent.
+		Parameters:
+			project -- The project that the instrument was added to.
+			instrument -- The instrument that was added.
 		"""
-		children = self.instrumentBox.get_children()
-		orderCounter = 0
-		for instr in self.project.instruments:
-			#Find the InstrumentView that matches instr:
-			iv = None
-			for ident, instrV in self.views:
-				if instrV.instrument is instr:
-					iv = instrV
-					break
-			#If there is no InstrumentView for instr, create one:
-			if not iv:
-				iv = InstrumentViewer.InstrumentViewer(self.project, instr, self, self.mainview, self.small)
-				# if this is mix view then add parent (CompactMixView) as listener
-				# otherwise add self
-				if self.mixView:
-					instr.AddListener(self.mixView)
-				else:
-					instr.AddListener(self)
-				#Add it to the views
-				self.views.append((instr.id, iv))
-				iv.headerAlign.connect("size-allocate", self.UpdateSize)
-			
-			if iv not in children:
-				#Add the InstrumentView to the VBox
-				self.instrumentBox.pack_start(iv, False, False)
-			else:
-				#If the InstrumentView has already been added, just move it
-				self.instrumentBox.reorder_child(iv, orderCounter)
+		instrViewer = InstrumentViewer.InstrumentViewer(project, instrument, self, self.mainview, self.small)
+		
+		#Add it to the views
+		self.views.append((instrument.id, instrViewer))
+		instrViewer.headerAlign.connect("size-allocate", self.UpdateSize)
+		
+		self.instrumentBox.pack_start(instrViewer, False, False)
+		instrViewer.show_all()
+		
+		self.ForceUpdateSize()
+	
+	#_____________________________________________________________________
+	
+	def OnInstrumentRemoved(self, project, instrument):
+		"""
+		Callback for when an instrument is removed from the project.
+		
+		Parameters:
+			project -- The project that the instrument was removed from.
+			instrument -- The instrument that was removed.
+		"""
+		for ID, instrViewer in self.views:
+			if ID == instrument.id:
+				if instrViewer.parent:
+					self.instrumentBox.remove(instrViewer)
+				instrViewer.Destroy()
+				self.views.remove((ID, instrViewer))
+				break
 				
-			#Make sure the InstrumentView is visible:
-			iv.show()
-			
-			orderCounter += 1
+		self.ForceUpdateSize()
+	
+	#_____________________________________________________________________
+	
+	def OnInstrumentReordered(self, project, instrument):
+		"""
+		Callback for when an instrument's position in the project has changed.
 		
-		removeList = []
-		#self.views is up to date now
-		for ident, iv in self.views:
-			#check if instrument has been deleted
-			if not iv.instrument in self.project.instruments:
-				if iv in children:
-					self.instrumentBox.remove(iv)
-				iv.Destroy()
-				removeList.append((ident, iv))
-			else:
-				iv.Update() #Update non-deleted instruments
+		Parameters:
+			project -- The project that the instrument was changed on.
+			instrument -- The instrument that was reordered.
+		"""
+		for ID, instrViewer in self.views:
+			if ID == instrument.id:
+				if instrViewer.parent:
+					pos = self.project.instruments.index(instrument)
+					self.instrumentBox.reorder_child(instrViewer, pos)
+					instrViewer.show_all()
+				break
 		
-		#remove all the unused ones so the garbage collector can clean then up
-		for tuple_ in removeList:
-			self.views.remove(tuple_)
-		del removeList
-		
-		if len(self.views) > 0:
+	
+	#_____________________________________________________________________
+	
+	def ForceUpdateSize(self):
+		"""
+		Force and update of a header size to match the size of the first instrument viewer.
+		"""
+		if self.views:
 			self.UpdateSize(None, self.views[0][1].headerAlign.get_allocation())
 		else:
 			self.UpdateSize(None, None)
-		self.show_all()
 	
 	#_____________________________________________________________________
-		
+	
 	def UpdateSize(self, widget=None, size=None):
 		"""
-		Called during update() to re-align the timeline and scrollbars
-		with the start of the event lane since the instrument width may
-		have been altered.
+		This method will re-align the timeline and scrollbars
+		with the start of the event lane since the instrument 
+		width may have been altered.
 		"""
 		#find the width of the instrument headers (they should all be the same size)
 		if size:
@@ -311,7 +315,7 @@ class RecordingView(gtk.Frame):
 		Globals.INSTRUMENT_HEADER_WIDTH = tempWidth
 		
 		#align timeline and scrollbar
-		self.timelinebar.Update()
+		self.timelinebar.UpdateSize()
 		self.al.set_padding(0, 0, tempWidth, 0)
 	
 	#_____________________________________________________________________
@@ -358,7 +362,7 @@ class RecordingView(gtk.Frame):
 			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		tmp = self.project.viewScale * 4. / 5.
-		#setting the value will trigger the gtk event and call OnZoom for us.
+		#setting the value will trigger the "value-changed" signal and call OnZoom for us.
 		self.zoomSlider.set_value(tmp)
 		
 	#_____________________________________________________________________
@@ -383,8 +387,24 @@ class RecordingView(gtk.Frame):
 			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		tmp = self.project.viewScale * 1.25
-		#setting the value will trigger the gtk event and call OnZoom for us.
+		#setting the value will trigger the "value-changed" signal and call OnZoom for us.
 		self.zoomSlider.set_value(tmp)
+
+	#_____________________________________________________________________
+		
+	def OnZoomReset(self, widget, mouse):
+		"""
+		Calls OnZoom when the user resets the zoom to the default by right-clicking.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			mouse -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		if mouse.button == 3:
+			tmp = (self.ZOOM_MAX_SCALE - self.ZOOM_MIN_SCALE) / 2
+			#setting the value will trigger the "value-changed" signal and call OnZoom for us.
+			self.zoomSlider.set_value(tmp)
+			return True
 		
 	#_____________________________________________________________________
 
@@ -417,9 +437,10 @@ class RecordingView(gtk.Frame):
 		"""
 		# If we're here then we're out of bounds of anything else
 		# So we should clear any selected events
+		if mouse.type == gtk.gdk._2BUTTON_PRESS or mouse.type == gtk.gdk._3BUTTON_PRESS:
+			return True
 		self.project.ClearEventSelections()
 		self.project.SelectInstrument(None)
-		self.Update()
 		
 	#_____________________________________________________________________
 	
@@ -438,22 +459,6 @@ class RecordingView(gtk.Frame):
 			
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None, *extra):
-		"""
-		Called when a change of state is signalled by any of the
-		objects this view is 'listening' to.
-		
-		Parameters:
-			obj -- object changing state. *CHECK*
-			change -- the change which has occured.
-			extra -- extra parameters passed by the caller.
-		"""
-		#don't update on volume change because it happens very often
-		if change != "volume":
-			self.Update()
-		
-	#_____________________________________________________________________
-	
 	def OnDragDataReceived(self, widget, context, x, y, selection, targetType, time):
 		"""
 		Called when the user releases MOUSE1, finishing a drag and drop
@@ -478,7 +483,6 @@ class RecordingView(gtk.Frame):
 		self.project.AddInstrumentAndEvents(uris, True) # True: copy
 		
 		context.finish(True, False, time)
-		self.Update()
 		return True
 	
 	#_____________________________________________________________________
@@ -501,4 +505,30 @@ class RecordingView(gtk.Frame):
 		return True
 	
 	#_____________________________________________________________________
+	
+	def ChangeSize(self, small):
+		"""
+		Alters the size of the instrument lanes and removes the zoom buttons.
+		
+		Parameters:
+			small -- True if changing to small. Otherwise False.
+		"""
+		#if the requested size has not changed then quit
+		if small == self.small:
+			return
+		self.small = small
+		children = self.instrumentBox.get_children()
+		for instrView in children:
+			instrView.ChangeSize(small)
+		if self.small:
+			self.inbutton.hide()
+			self.outbutton.hide()
+			self.zoomSlider.hide()
+		else:
+			self.inbutton.show()
+			self.outbutton.show()
+			self.zoomSlider.show()
+			
+	#____________________________________________________________________	
+
 #=========================================================================
