@@ -22,13 +22,7 @@ class TimeLine(gtk.DrawingArea):
 	This class handles drawing the time line display. The time line is part of the
 	TimeLineBar. TimeLine displays the time in minutes/seconds (MODE_HOURS_MINS_SECS)
 	or bars and beats (MODE_BARS_BEATS). These modes are set in project.transport.
-		
-	To improve performance, the line isn't being constructed on each call of OnDraw. It
-	is saved into self.savedLine as a gtk.gdk.Image. A new savedLine is constructed when
-		- there is no savedLine
-		- or project.transport.RedrawTimeLine is True
-		- or project.RedrawTimeLine is True
-		
+	
 	When the time line is constructed in MODE_HOURS_MINS_SECS, it dynamically adjusts
 	its scale to the project.viewScale. MODE_BARS_BEATS does not support this (yet).
 	"""
@@ -72,8 +66,12 @@ class TimeLine(gtk.DrawingArea):
 		self.mainview = mainview
 		
 		# Listen for changes in the project and the TransportManager
-		self.project.transport.AddListener(self)
-		self.project.AddListener(self)
+		self.project.transport.connect("transport-mode", self.OnTransportMode)
+		self.project.transport.connect("position", self.OnTransportPosition)
+		self.project.connect("bpm", self.OnProjectTimelineChange)
+		self.project.connect("time-signature", self.OnProjectTimelineChange)
+		self.project.connect("view-start", self.OnProjectTimelineChange)
+		self.project.connect("zoom", self.OnProjectTimelineChange)
 		
 		self.height = 44
 		self.buttonDown = False
@@ -221,19 +219,29 @@ class TimeLine(gtk.DrawingArea):
 				
 				# x is now at the pixel-position of the first beat after the viewStart
 				beat += 1
+
+			spacing = (60. / self.project.bpm) * self.project.viewScale
+
+			if self.project.meter_denom == 8 and (self.project.meter_nom % 3) == 0 and self.project.meter_nom != 3:
+				# Compound time signature, so beats are really 1 dotted note (3 1/8 notes)
+				beats_per_bar = self.project.meter_nom / 3
+				spacing *= 3
+			else:
+				# Simple meter
+				beats_per_bar = self.project.meter_nom
 		
 			while x < self.get_allocation().width:
 				# Draw the beat/bar divisions
 				ix = int(x)
-				
-				if beat % self.project.meter_nom:
+
+				if beat % beats_per_bar:
 					lineHeight = int(self.get_allocation().height/1.2)
 				else:
 					lineHeight = int(self.get_allocation().height/2)
 					
 					# Draw the bar number
 					context.set_source_rgb(*self._TEXT_RGB)
-					number = (beat / self.project.meter_nom)+1
+					number = (beat / beats_per_bar)+1
 					
 					# TODO: small hack to fix a problem with the numbers not being
 					#       properly centered
@@ -251,8 +259,8 @@ class TimeLine(gtk.DrawingArea):
 				context.stroke()
 					
 				beat += 1
-				
-				x += (60. / self.project.bpm) * self.project.viewScale
+
+				x += spacing
 		else:
 			# Working in milliseconds here. Using seconds gives modulus problems because they're floats
 			viewScale = self.project.viewScale / 1000.
@@ -327,63 +335,72 @@ class TimeLine(gtk.DrawingArea):
 		
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None, *extra):
-		""" 
-		Called when there is a change of state in transport	manager or project. 
-		Could be one of
-			*  Mode changed from bars/beats to minutes or vice versa
-			(requires a complete redraw of timeline)
-			*  Change in playing position -only needs partial redraw
-			*  Project change e.g. a scroll or zoom change
-			(requires a complete redraw of timeline)
-				
-		Parameters:
-			obj -- an object to inform when this method is called.
-			change -- the change which has occured.
-			extra -- the extra parameters to be passed.
+	def OnProjectTimelineChange(self, project):
 		"""
+		Callback for signal when time signature, zoom level,
+		bpm or view start of the project change. All of these things
+		effect the way that the timeline is drawn.
+		
+		Parameters:
+			project -- the project instance that send the signal.
+		"""
+		self.queue_draw()
+		
+	#_____________________________________________________________________
+	
+	def OnTransportMode(self, transportManager, mode):
+		"""
+		Callback for signal when the transport mode changes.
+		
+		Parameters:
+			transportManager -- the TransportManager instance that send the signal.
+			mode -- the mode type that the transport changed to.
+		"""
+		self.queue_draw()
+	
+	#_____________________________________________________________________
+	
+	def OnTransportPosition(self, transportManager, extraString):
+		"""
+		Callback for signal when the transport position changes.
+		
+		Parameters:
+			transportManager -- the TransportManager instance that send the signal.
+			extraString -- a string specifying the extra action details. i.e. "stop-action"
+					means that the position changed because the user hit stop.
+		"""
+	
 		#if the timeline is not currently on screen then quit
 		if not self.window:
 			return
 		
-		#redraw timeline if needed
-		if change in ("transport-mode", "bpm", "time-signature", "zoom", "view-start"):
-			# delete the cached image so that it will be redrawn
-			self.savedLine = None
-			self.timelinebar.Update()
-			#for bpm change re-align timeline with event lane
-			if change == "bpm":
-				self.timelinebar.projectview.UpdateSize()
+		# The left and right sides of the viewable area
+		rightPos = self.project.viewStart + self.timelinebar.projectview.scrollRange.page_size
+		leftPos = self.project.viewStart
+		currentPos = self.project.transport.GetPosition()
 		
-		# The next section is the autoscroll when the position goes off the screen
-		elif change == "position":
-			# The left and right sides of the viewable area
-			rightPos = self.project.viewStart + self.timelinebar.projectview.scrollRange.page_size
-			leftPos = self.project.viewStart
-			currentPos = self.project.transport.GetPosition()
+		# Check if the playhead was recently viewable (don't force it in view if it wasn't previously in view)
+		# Don't autoscroll if "stop-action" is send in extra because that means the 
+		# user just hit stop and did not purposely change the position.
+		if "stop-action" != extraString:
+			if leftPos < self.project.transport.PrevPosition < rightPos:
+				if currentPos > rightPos:
+					# now the playhead has moved off to the right, so force the scroll in that direction
+					self.timelinebar.projectview.SetViewPosition(rightPos)
 			
-			# Check if the playhead was recently viewable (don't force it in view if it wasn't previously in view)
-			# Don't autoscroll if "stop-action" is send in extra because that means the 
-			# user just hit stop and did not purposely change the position.
-			if not "stop-action" in extra:
-				if leftPos < self.project.transport.PrevPosition < rightPos:
-					if currentPos > rightPos:
-						# now the playhead has moved off to the right, so force the scroll in that direction
-						self.timelinebar.projectview.SetViewPosition(rightPos)
-				
-					elif currentPos < leftPos:
-						#if playhead is beyond leftmost position then force scroll and quit
-						leftPos = max(0, leftPos - self.timelinebar.projectview.scrollRange.page_size)
-						self.timelinebar.projectview.SetViewPosition(leftPos)
-		
-			x1 = round((self.project.transport.PrevPosition - self.project.viewStart) * self.project.viewScale)
-			x2 = round((self.project.transport.position - self.project.viewStart) * self.project.viewScale)
-		
-			self.queue_draw_area(int(x1)-1, 0, 3, self.get_allocation().height)
-			self.queue_draw_area(int(x2)-1, 0, 3, self.get_allocation().height)
-		
+				elif currentPos < leftPos:
+					#if playhead is beyond leftmost position then force scroll and quit
+					leftPos = max(0, leftPos - self.timelinebar.projectview.scrollRange.page_size)
+					self.timelinebar.projectview.SetViewPosition(leftPos)
+	
+		x1 = round((self.project.transport.PrevPosition - self.project.viewStart) * self.project.viewScale)
+		x2 = round((self.project.transport.position - self.project.viewStart) * self.project.viewScale)
+	
+		self.queue_draw_area(int(x1)-1, 0, 3, self.get_allocation().height)
+		self.queue_draw_area(int(x2)-1, 0, 3, self.get_allocation().height)
+	
 	#_____________________________________________________________________
-		
+	
 	def onMouseDown(self, widget, event):
 		"""
 		Called when a mouse button is clicked.
