@@ -72,7 +72,12 @@ class TimeLineBar(gtk.Frame):
 		self.bpmlabel.set_padding(5, 5)
 		self.bpmeventbox.add(self.bpmlabel)
 		self.bpmframe.add(self.bpmeventbox)
-		self.bpmeditPacked = False
+		
+		self.bpmedit = gtk.SpinButton()
+		self.bpmedit.set_range(1, 400)
+		self.bpmedit.set_increments(1, 5)
+		self.bpmedit.set_value(self.project.bpm)
+		self.bpmedit.connect("activate", self.OnAcceptEditBPM)
 
 		self.sigeventbox = gtk.EventBox()
 		self.sigeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
@@ -100,6 +105,10 @@ class TimeLineBar(gtk.Frame):
 		self.sigeventbox.connect("button_press_event", self.OnEditSig)
 		self.sigeventbox.connect("enter_notify_event", self.OnMouseMoveTimeSig)
 		self.sigeventbox.connect("leave_notify_event", self.OnMouseMoveTimeSig)
+		
+		self.project.connect("bpm", self.OnProjectBPMChange)
+		self.project.connect("time-signature", self.OnProjectSigChange)
+		self.project.connect("click-track", self.OnProjectClickTrackChange)
 
 		self.sigDialog = None
 		
@@ -117,7 +126,7 @@ class TimeLineBar(gtk.Frame):
 		self.alignment.add(self.headerhbox)
 		self.hbox.pack_start(self.alignment, False, False)
 		self.add(self.hbox)
-		self.headerhbox.connect("check-resize", self.projectview.Update)
+		self.headerhbox.connect("check-resize", self.projectview.ForceUpdateSize)
 		self.connect("size-allocate", self.OnAllocate)
 		self.hbox.pack_start(self.timeline)	
 
@@ -137,42 +146,87 @@ class TimeLineBar(gtk.Frame):
 
 	#_____________________________________________________________________
 	
-	def Update(self):
+	def UpdateSize(self):
 		""" 
-		Updates the contents TimeLineBar, updating the values in the beats per
+		Updates the size of the header box contents TimeLineBar, updating the values in the beats per
 		minute box and time signature box, as well as updating the click button
 		sensitivity and instrument header width.
 		"""
-		if not self.Updating:
-			instrumentviews=[]
-			if self.mainview.recording:
-				instrumentviews+=self.mainview.recording.views
-			if self.mainview.compactmix:
-				instrumentviews+=self.mainview.compactmix.projectview.views
+		if self.Updating or not self.mainview.workspace:
+			return
+		
+		instrumentViews = self.mainview.workspace.recordingView.views
 
-			self.Updating = True
-			maxwidth=self.headerhbox.size_request()[0]
+		self.Updating = True
+		maxwidth = self.headerhbox.size_request()[0]
+
+		for ident, iv in instrumentViews:  #self.mainview.recording.views:
+			if iv.instrument in iv.mainview.project.instruments:
+				if iv.headerBox.size_request()[0] > maxwidth:
+					maxwidth = iv.headerBox.size_request()[0]
+
+		for ident, iv in instrumentViews:  #self.mainview.recording.views:
+			if iv.headerAlign.size_request()[0] != (maxwidth+2):
+				iv.ResizeHeader(maxwidth+2)
+		
+		Globals.INSTRUMENT_HEADER_WIDTH = maxwidth + 2
+		
+		self.alignment.set_padding(0, 0, 0, maxwidth - self.headerhbox.size_request()[0])
+		
+		self.Updating = False
+		
+	#_____________________________________________________________________
 	
-			for ident, iv in instrumentviews:  #self.mainview.recording.views:
-				if iv.instrument in iv.mainview.project.instruments:
-					if iv.headerBox.size_request()[0] > maxwidth:
-						maxwidth = iv.headerBox.size_request()[0]
-
-			for ident, iv in instrumentviews:  #self.mainview.recording.views:
-				if iv.headerAlign.size_request()[0] != (maxwidth+2):
-					iv.ResizeHeader(maxwidth+2)
-			
-			Globals.INSTRUMENT_HEADER_WIDTH = maxwidth + 2
-			
-			self.alignment.set_padding(0, 0, 0, maxwidth - self.headerhbox.size_request()[0])
-
-			self.clickbutton.set_active(self.project.clickEnabled)
-			self.OnAcceptEditBPM()
-			self.UpdateSigLabel()
-			self.timeline.queue_draw()
-			
-			self.Updating = False
-			
+	def OnProjectBPMChange(self, project):
+		"""
+		Callback for when the BPM of the project changes. This method 
+		will update the contents of the beats per minute box.
+		
+		Parameters:
+			project -- The project that send the signal.
+		"""
+		#Do this outside the if statement so that it gets updated if someone else changes the bpm
+		self.bpmlabel.set_use_markup(True)
+		self.bpmlabel.set_markup("<span foreground='%s'><b>%d</b></span>" % (self.fontColor, self.project.bpm))
+		self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
+		
+		self.projectview.UpdateSize()
+		
+	#_____________________________________________________________________
+	
+	def OnProjectSigChange(self, project):
+		"""
+		Callback for when the BPM of the project changes. This method 
+		will update the contents of the time signature label.
+		
+		Parameters:
+			project -- The project that send the signal.
+		"""
+		self.siglabel.set_use_markup(True)
+		self.siglabel.set_markup("<span foreground='%s'><b>%d/%d</b></span>" % (self.fontColor, self.project.meter_nom, self.project.meter_denom))
+		self.projectview.UpdateSize()
+		
+	#_____________________________________________________________________
+	
+	def OnProjectClickTrackChange(self, project):
+		"""
+		Callback for when the click track of the project it turned on
+		or shut off. This method will update the button and tooltip state.
+		
+		Parameters:
+			project -- The project that send the signal.
+		"""
+		self.Updating = True
+		self.clickbutton.set_active(self.project.clickEnabled)
+		self.Updating = False
+		
+		if self.project.clickEnabled:
+			tooltip = _("Turn click track off")
+		else:
+			tooltip = _("Turn click track on")
+		
+		self.clicktip.set_tip(self.clickbutton, tooltip, None)
+		
 	#_____________________________________________________________________
 	
 	def OnEditBPM(self, widget, event):
@@ -186,24 +240,40 @@ class TimeLineBar(gtk.Frame):
 		"""
 		if event.type == gtk.gdk.BUTTON_PRESS:
 			startWidth = self.headerhbox.size_request()[0]
-			self.bpmframe.remove(self.bpmeventbox)
-						
-			self.bpmedit = gtk.SpinButton()
-			self.bpmedit.set_range(1, 400)
-			self.bpmedit.set_increments(1, 5)
-			self.bpmedit.set_value(self.project.bpm)
-			self.bpmedit.connect("activate", self.OnAcceptEditBPM)
-
-			self.bpmframe.add(self.bpmedit)
-			self.bpmedit.show()
-			self.bpmedit.grab_focus()
-			self.bpmeditPacked = True
+			
+			if self.bpmeventbox.parent:
+				self.bpmframe.remove(self.bpmeventbox)
+				self.bpmframe.add(self.bpmedit)
+				self.bpmedit.show()
+				self.bpmedit.grab_focus()
+			
 			#adjust padding so that the timeline event lanes still line up
 			newWidth = self.headerhbox.size_request()[0]
 			padding = self.alignment.get_padding()
 			self.alignment.set_padding(0, 0, 0, padding[3] - (newWidth -startWidth))
 
 	#_____________________________________________________________________
+	
+	def OnAcceptEditBPM(self, widget=None):
+		"""
+		Called when the user finishes editing the beats per minute box.
+		This method then updates the beats per minute value to the value the user 
+		enters.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		
+		if self.bpmedit.parent:
+			self.bpmframe.remove(self.bpmedit)
+			self.bpmframe.add(self.bpmeventbox)
+			self.bpmframe.show_all()
+			
+			newbpm = self.bpmedit.get_value_as_int()
+			self.project.SetBPM(float(newbpm))
+			self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
+		
+	#_____________________________________________________________________
 
 	def OnEditSig(self, widget, event):
 		"""
@@ -237,7 +307,7 @@ class TimeLineBar(gtk.Frame):
 				self.comboValue.set_active(int(log(self.project.meter_denom, 2)))
 			else:
 				self.sigDialog.present()
-							
+			
 	#_____________________________________________________________________
 	
 	def OnNomValueChange(self, spinButton):
@@ -277,48 +347,6 @@ class TimeLineBar(gtk.Frame):
 
 	#_____________________________________________________________________
 	
-	def UpdateSigLabel(self):
-		"""
-		Updates the time signature label in the main Jokosher window.
-		"""
-		self.siglabel.set_use_markup(True)
-		self.siglabel.set_markup("<span foreground='%s'><b>%d/%d</b></span>" % (self.fontColor, self.project.meter_nom, self.project.meter_denom))
-		self.projectview.UpdateSize()
-		
-	#_____________________________________________________________________
-	
-	def OnAcceptEditBPM(self, widget=None):
-		"""
-		Called when the user finishes editing the beats per minute box.
-		This method then updates the beats per minute value to the value the user 
-		enters and then writes that change to disk if the user saves the project. 
-		If anything but OnEditBPM calls this method, it will update the contents of the beats per minute box.
-		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-		"""
-		if self.bpmeditPacked:
-			self.bpmframe.remove(self.bpmedit)
-			#FIXME: find a better way to do project.PrepareClick() it doesn't take a really long time with large bpm
-			newbpm = self.bpmedit.get_value_as_int()
-			
-			self.project.SetBPM(float(newbpm))
-			self.project.PrepareClick()
-			
-			self.bpmframe.add(self.bpmeventbox)
-			self.bpmedit.destroy()
-			self.bpmframe.show_all()
-			self.bpmeditPacked = False
-		
-		#Do this outside the if statement so that it gets updated if someone else changes the bpm
-		self.bpmlabel.set_use_markup(True)
-		self.bpmlabel.set_markup("<span foreground='%s'><b>%d</b></span>" % (self.fontColor, self.project.bpm))
-		self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
-			
-		self.projectview.UpdateSize()
-		
-	#_____________________________________________________________________
-	
 	def OnMouseMoveBPM(self, widget, event):
 		"""
 		Called when the mouse pointer enters or leaves the beats per minute box.
@@ -361,17 +389,12 @@ class TimeLineBar(gtk.Frame):
 
 	def OnClick(self, widget):
 		"""
-		Called when the click button is clicked.
-		This method will also set the clicked button to appear pressed in if clicked.
-		If the click button is clicked while in a 'pressed in' state. It will appear as it did originally.
+		Called when the click button is clicked. This method will call the project
+		to turn on or shut off the click track.
 		""" 
-		if widget.get_active() == True:
-			self.project.EnableClick()
-			self.clicktip.set_tip(self.clickbutton, _("Turn click track off"), None)
-		if widget.get_active() == False:
-			self.project.DisableClick()
-			self.clicktip.set_tip(self.clickbutton, _("Turn click track on"), None)
+		if not self.Updating:
+			self.project.SetClickEnabled(widget.get_active())
 			
 	#_____________________________________________________________________
-		
+	
 #=========================================================================
