@@ -10,11 +10,18 @@
 #-------------------------------------------------------------------------------
 
 import xml.dom.minidom as xml
-import math, os.path
+import math, os.path, sys
 import gtk, gobject
 from subprocess import call
 import Globals
 
+import gst
+if gst.pygst_version >= (0, 10, 10):
+    import gst.pbutils
+
+# the highest range in decibels there can be between any two levels
+DECIBEL_RANGE = 80
+
 #_____________________________________________________________________
 
 def OpenExternalURL(url, message, parent):
@@ -104,6 +111,44 @@ def DbToFloat(f):
 
 #_____________________________________________________________________
 
+def CalculateAudioLevel(channelLevels):
+	"""
+	Calculates an average for all channel levels.
+	
+	Parameters:
+		channelLevels -- list of levels from each channel.
+		
+	Returns:
+		an average level, also taking into account negative infinity numbers,
+		which will be discarded in the average.
+	"""
+	if not channelLevels:
+		return 0
+
+	negInf = float("-inf")
+	peaktotal = 0
+	for peak in channelLevels:
+		#if peak > 0.001:
+		#	print channelLevels
+		#don't add -inf values cause 500 + -inf is still -inf
+		if peak != negInf:
+			peaktotal += peak
+		else:
+			peaktotal -= DECIBEL_RANGE
+	
+	peaktotal /= len(channelLevels)
+	
+	peaktotal += DECIBEL_RANGE
+	#convert to an integer
+	peaktotal = min(peaktotal, DECIBEL_RANGE)
+	peaktotal = max(peaktotal, -DECIBEL_RANGE)
+	peakint = int((peaktotal / DECIBEL_RANGE) * sys.maxint)
+
+	return peakint
+
+
+#_____________________________________________________________________
+
 def floatRange(start, end=None, inc=None):
 	"""
 	A range function capable of performing float increments.
@@ -333,3 +378,22 @@ def StoreVariableToNode(value, node, typeAttr="type", valueAttr="value"):
 	node.setAttribute(valueAttr, str(value))
 
 #_____________________________________________________________________
+
+def HandleGstPbutilsMissingMessage(message, callback, x_window_id=0):
+	# pbutils was wrapped in 0.10.10
+	if gst.pygst_version < (0, 10, 10):
+		return False
+
+	#self._installing_plugins = True
+	
+	detail = gst.pbutils.missing_plugin_message_get_installer_detail(message)
+	ctx = gst.pbutils.InstallPluginsContext()
+	Globals.debug(detail)
+	if x_window_id:
+		ctx.set_x_id(x_window_id)
+	
+	ret = gst.pbutils.install_plugins_async([detail], ctx, callback)
+	return True
+
+#_____________________________________________________________________
+
